@@ -14,19 +14,30 @@
    Leave the following blank line there!!  Autoheader needs it.  */
 
 
+#undef ENABLE_MP
+#undef ENABLE_NLS
+#undef HAVE_CATGETS
 #undef HAVE_DIRENT_H
 #undef HAVE_DOPRNT
+#undef HAVE_GETTEXT
 #undef HAVE_IPC_H
+#undef HAVE_LC_MESSAGES
+#undef HAVE_MMAP
 #undef HAVE_NDIR_H
+#undef HAVE_PROGRESSIVE_JPEG
 #undef HAVE_PUTENV
+#undef HAVE_RINT
 #undef HAVE_SHM_H
+#undef HAVE_STPCPY
 #undef HAVE_SYS_DIR_H
 #undef HAVE_SYS_NDIR_H
+#undef HAVE_SYS_PARAM_H
 #undef HAVE_SYS_SELECT_H
 #undef HAVE_SYS_TIME_H
+#undef HAVE_SYS_TIMES_H
 #undef HAVE_UNISTD_H
 #undef HAVE_VPRINTF
-#undef HAVE_XSHM_H
+#undef HAVE_VSNPRINTF
 
 #undef IPC_RMID_DEFERRED_RELEASE
 
@@ -36,6 +47,8 @@
 #undef RAND_FUNC
 #undef SRAND_FUNC
 
+#undef USE_PTHREADS
+
 
 /* Leave that blank line there!!  Autoheader needs it.
    If you're adding to this file, keep in mind:
