@@ -23,6 +23,7 @@
 #include "gimpcontext.h"
 #include "tool_options.h"
 #include "channel.h"
+#include "cursorutil.h"
 
 #include "toolsF.h"
 
@@ -47,7 +48,8 @@ typedef enum
   SELECTION_REPLACE   = REPLACE,
   SELECTION_INTERSECT = INTERSECT,
   SELECTION_MOVE_MASK,
-  SELECTION_MOVE
+  SELECTION_MOVE,
+  SELECTION_ANCHOR
 } SelectOps;
 
 /*  The possibilities for where the cursor lies  */
@@ -71,6 +73,10 @@ struct _Tool
   void      *gdisp_ptr;     /*  pointer to currently active gdisp           */
   void      *drawable;      /*  pointer to the tool's current drawable      */
 
+  gboolean   toggled;       /*  Bad hack to let the paint_core show the     */
+                            /*  right toggle cursors                        */
+
+
   void      *private;       /*  Tool-specific information                   */
 
   /*  Action functions  */
@@ -95,6 +101,7 @@ struct _ToolInfo
 
   gchar    **icon_data;
   GdkPixmap *icon_pixmap;
+  GdkBitmap *icon_mask;
 
   gchar     *tool_desc;
   gchar     *private_tip;
@@ -108,6 +115,9 @@ struct _ToolInfo
   GtkWidget *tool_widget;
 
   GimpContext *tool_context;
+
+  BitmapCursor tool_cursor;
+  BitmapCursor toggle_cursor;
 };
 
 /*  Global Data Structures  */
@@ -136,11 +146,8 @@ void     tool_options_dialog_free  (void);
 
 gchar  * tool_active_PDB_string    (void);
 
-/* don't unref this pixmap, it is static! */
+/* don't unref this pixmaps, they are static! */
 GdkPixmap * tool_get_pixmap        (ToolType     tool_type);
+GdkBitmap * tool_get_mask          (ToolType     tool_type);
 
 #endif  /*  __TOOLS_H__  */
-
-
-
-
