@@ -126,6 +126,7 @@
 				 * pretty obscure Win32 compilation issues.
 				 */
 #include <setjmp.h>
+#include <signal.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -142,7 +143,6 @@
 
 #include "libgimp/stdplugins-intl.h"
 
-#include <signal.h>
 
 #define SCALE_WIDTH         125
 
@@ -229,7 +229,6 @@ static void   add_menu_item             (GtkWidget *menu,
 					 guint op_no,
 					 GtkSignalFunc callback);
 
-static void   init_gtk                   (void);
 static gint   save_dialog                (void);
 
 static void   save_close_callback        (GtkWidget *widget,
@@ -289,11 +288,11 @@ query (void)
   {
     { PARAM_INT32,    "run_mode",     "Interactive, non-interactive" },
     { PARAM_STRING,   "filename",     "The name of the file to load" },
-    { PARAM_STRING,   "raw_filename", "The name of the file to load" },
+    { PARAM_STRING,   "raw_filename", "The name of the file to load" }
   };
   static GParamDef load_return_vals[] =
   {
-    { PARAM_IMAGE,   "image",         "Output image" },
+    { PARAM_IMAGE,   "image",         "Output image" }
   };
   static gint nload_args        = sizeof (load_args) / sizeof (load_args[0]);
   static gint nload_return_vals = (sizeof (load_return_vals) /
@@ -318,8 +317,6 @@ query (void)
   };
   static gint nsave_args = sizeof (save_args) / sizeof (save_args[0]);
 
-  INIT_I18N();
-
   gimp_install_procedure ("file_jpeg_load",
                           "loads files in the JPEG file format",
                           "loads files in the JPEG file format",
@@ -338,7 +335,7 @@ query (void)
                           "Spencer Kimball, Peter Mattis & others",
                           "Spencer Kimball & Peter Mattis",
                           "1995-1999",
-                          "<Save>/Jpeg",
+                          "<Save>/JPEG",
 			  "RGB*, GRAY*",
                           PROC_PLUG_IN,
                           nsave_args, 0,
@@ -347,7 +344,7 @@ query (void)
   gimp_register_magic_load_handler ("file_jpeg_load",
 				    "jpg,jpeg",
 				    "",
-				    "6,string,JFIF");
+				    "6,string,JFIF,6,string,Exif");
   gimp_register_save_handler       ("file_jpeg_save",
 				    "jpg,jpeg",
 				    "");
@@ -368,7 +365,7 @@ run (gchar   *name,
   gint32        orig_image_ID;
   gint32        display_ID = -1;
 #ifdef GIMP_HAVE_PARASITES
-  Parasite     *parasite;
+  GimpParasite *parasite;
 #endif /* GIMP_HAVE_PARASITES */
   gint          err;
   GimpExportReturnType export = EXPORT_CANCEL;
@@ -409,7 +406,7 @@ run (gchar   *name,
 	{
 	case RUN_INTERACTIVE:
 	case RUN_WITH_LAST_VALS:
-	  init_gtk ();
+	  gimp_ui_init ("jpeg", FALSE);
 	  export = gimp_export_image (&image_ID, &drawable_ID, "JPEG", 
 				      (CAN_HANDLE_RGB |
 				       CAN_HANDLE_GRAY));
@@ -442,7 +439,7 @@ run (gchar   *name,
       if (parasite) 
 	{
 	  image_comment = g_strdup (parasite->data);
-	  parasite_free (parasite);
+	  gimp_parasite_free (parasite);
 	}
 #endif /* GIMP_HAVE_PARASITES */
       if (!image_comment) 
@@ -479,7 +476,7 @@ run (gchar   *name,
 	      jsvals.restart     = ((JpegSaveVals *)parasite->data)->restart;
 	      jsvals.dct         = ((JpegSaveVals *)parasite->data)->dct;
 	      jsvals.preview     = ((JpegSaveVals *)parasite->data)->preview;
-	      parasite_free(parasite);
+	      gimp_parasite_free (parasite);
 	    }
 #endif /* GIMP_HAVE_PARASITES */
 
@@ -560,7 +557,7 @@ run (gchar   *name,
 	      jsvals.restart     = ((JpegSaveVals *)parasite->data)->restart;
 	      jsvals.dct         = ((JpegSaveVals *)parasite->data)->dct;
 	      jsvals.preview     = FALSE;
-	      parasite_free(parasite);
+	      gimp_parasite_free (parasite);
 	    }
 #endif /* GIMP_HAVE_PARASITES */
 	  break;
@@ -605,18 +602,19 @@ run (gchar   *name,
       gimp_image_parasite_detach (orig_image_ID, "gimp-comment");
       if (strlen (image_comment)) 
 	{
-	  parasite = parasite_new ("gimp-comment",
-				   PARASITE_PERSISTENT,
-				   strlen (image_comment) + 1,
-				   image_comment);
+	  parasite = gimp_parasite_new ("gimp-comment",
+					GIMP_PARASITE_PERSISTENT,
+					strlen (image_comment) + 1,
+					image_comment);
 	  gimp_image_parasite_attach (orig_image_ID, parasite);
-	  parasite_free (parasite);
+	  gimp_parasite_free (parasite);
 	}
       gimp_image_parasite_detach (orig_image_ID, "jpeg-save-options");
       
-      parasite = parasite_new ("jpeg-save-options", 0, sizeof (jsvals), &jsvals);
+      parasite = gimp_parasite_new ("jpeg-save-options",
+				    0, sizeof (jsvals), &jsvals);
       gimp_image_parasite_attach (orig_image_ID, parasite);
-      parasite_free (parasite);
+      gimp_parasite_free (parasite);
 #endif /* Have Parasites */  
     }
   else
@@ -728,8 +726,8 @@ load_image (gchar        *filename,
 
 #ifdef GIMP_HAVE_PARASITES
   JpegSaveVals local_save_vals;
-  Parasite * volatile comment_parasite = NULL;
-  Parasite * volatile vals_parasite = NULL;
+  GimpParasite * volatile comment_parasite = NULL;
+  GimpParasite * volatile vals_parasite = NULL;
 #endif /* GIMP_HAVE_PARASITES */
 
   
@@ -794,8 +792,9 @@ load_image (gchar        *filename,
 	char *string = local_image_comments->str;
 	g_string_free (local_image_comments,FALSE);
 	local_image_comments = NULL;
-	comment_parasite = parasite_new ("gimp-comment", PARASITE_PERSISTENT,
-					 strlen (string) + 1, string);
+	comment_parasite = gimp_parasite_new ("gimp-comment",
+					      GIMP_PARASITE_PERSISTENT,
+					      strlen (string) + 1, string);
       } 
     else 
       {
@@ -822,8 +821,9 @@ load_image (gchar        *filename,
     local_save_vals.dct         = DEFAULT_DCT;
     local_save_vals.preview     = DEFAULT_PREVIEW;
   
-    vals_parasite = parasite_new ("jpeg-save-options", 0,
-				  sizeof (local_save_vals), &local_save_vals);
+    vals_parasite = gimp_parasite_new ("jpeg-save-options", 0,
+				       sizeof (local_save_vals),
+				       &local_save_vals);
   } 
 #endif /* GIMP_HAVE_PARASITES */
   
@@ -1043,20 +1043,22 @@ load_image (gchar        *filename,
 
   /* pw - Last of all, attach the parasites (couldn't do it earlier -
      there was no image. */
-  
+
+#ifdef GIMP_HAVE_PARASITES
   if (!preview) 
     {
       if (comment_parasite)
 	{
 	  gimp_image_parasite_attach (image_ID, comment_parasite);
-	  parasite_free (comment_parasite);
+	  gimp_parasite_free (comment_parasite);
 	}
       if (vals_parasite)
 	{
 	  gimp_image_parasite_attach (image_ID, vals_parasite);
-	  parasite_free (vals_parasite);
+	  gimp_parasite_free (vals_parasite);
 	}   
     }
+#endif /* GIMP_HAVE_PARASITES */
 
   return image_ID;
 }
@@ -1382,7 +1384,7 @@ save_image (char   *filename,
   jpeg_start_compress (&cinfo, TRUE);
 
   /* Step 4.1: Write the comment out - pw */
-  if (image_comment) 
+  if (image_comment && *image_comment)
     {
       jpeg_write_marker (&cinfo,
 			 JPEG_COM,
@@ -1552,20 +1554,6 @@ add_menu_item (GtkWidget        *menu,
   gtk_widget_show (menu_item);
 }
 
-static void 
-init_gtk (void)
-{
-  gchar **argv;
-  gint argc;
-
-  argc    = 1;
-  argv    = g_new (gchar *, 1);
-  argv[0] = g_strdup ("jpeg");
-
-  gtk_init (&argc, &argv);
-  gtk_rc_parse (gimp_gtkrc ());
-}
-
 static gint
 save_dialog (void)
 {
@@ -1599,8 +1587,8 @@ save_dialog (void)
   GtkWidget *prv_frame;
   GDrawableType dtype;
 
-  dlg = gimp_dialog_new (_("Save as Jpeg"), "jpeg",
-			 gimp_plugin_help_func, "filters/jpeg.html",
+  dlg = gimp_dialog_new (_("Save as JPEG"), "jpeg",
+			 gimp_standard_help_func, "filters/jpeg.html",
 			 GTK_WIN_POS_MOUSE,
 			 FALSE, TRUE, FALSE,
 
@@ -1631,7 +1619,7 @@ save_dialog (void)
   gtk_container_add (GTK_CONTAINER (prv_frame), vbox);
   gtk_widget_show (vbox);
 
-  preview = gtk_check_button_new_with_label (_("Preview (in Image Window)"));
+  preview = gtk_check_button_new_with_label (_("Preview (in image window)"));
   gtk_box_pack_start (GTK_BOX (vbox), preview, FALSE, FALSE, 0);
   gtk_signal_connect (GTK_OBJECT (preview), "toggled",
 		      GTK_SIGNAL_FUNC (gimp_toggle_button_update),
@@ -1711,7 +1699,7 @@ save_dialog (void)
 				   1, 64, 1, 1, 0.0);
   restart_markers_scale = gtk_hscale_new (GTK_ADJUSTMENT (scale_data));
 
-  restart = gtk_check_button_new_with_label (_("Restart Markers"));
+  restart = gtk_check_button_new_with_label (_("Restart markers"));
   gtk_table_attach (GTK_TABLE (table), restart, 0, 1, 2, 3,
 		    GTK_FILL, 0, 0, 0);
   gtk_signal_connect (GTK_OBJECT (restart), "toggled",
@@ -1719,7 +1707,7 @@ save_dialog (void)
 		      scale_data);
   gtk_widget_show (restart);
 
-  restart_markers_label = gtk_label_new (_("Restart Frequency (Rows):"));
+  restart_markers_label = gtk_label_new (_("Restart frequency (rows):"));
   gtk_misc_set_alignment (GTK_MISC (restart_markers_label), 1.0, 1.0);
   gtk_table_attach (GTK_TABLE (table), restart_markers_label, 0, 1, 3, 4,
 		    GTK_FILL | GTK_SHRINK, GTK_FILL, 0, 0);
@@ -1777,7 +1765,7 @@ save_dialog (void)
   gtk_widget_set_sensitive (progressive, FALSE);
 #endif
   
-  baseline = gtk_check_button_new_with_label (_("Force Baseline JPEG (Readable by all Decoders)"));
+  baseline = gtk_check_button_new_with_label (_("Force baseline JPEG (Readable by all decoders)"));
   gtk_table_attach (GTK_TABLE (table), baseline, 0, 3, 6, 7,
 		    GTK_FILL, 0, 0, 0);
   gtk_signal_connect (GTK_OBJECT (baseline), "toggled",
@@ -1818,7 +1806,7 @@ save_dialog (void)
 
   dct_menu = gtk_option_menu_new ();
  
-  label = gtk_label_new (_("DCT Method (Speed/Quality Tradeoff):"));
+  label = gtk_label_new (_("DCT method (Speed/quality tradeoff):"));
   gtk_misc_set_alignment (GTK_MISC (label), 1.0, 0.5);
   gtk_table_attach (GTK_TABLE (table), label, 0, 2, 8, 9,
 		    GTK_FILL | GTK_SHRINK, GTK_FILL, 0, 0);
@@ -1835,7 +1823,7 @@ save_dialog (void)
       gtk_widget_set_sensitive (subsmp_menu, FALSE);
     }
 
-  com_frame = gtk_frame_new (_("Image Comments"));
+  com_frame = gtk_frame_new (_("Image comments"));
   gtk_frame_set_shadow_type (GTK_FRAME (com_frame), GTK_SHADOW_ETCHED_IN);
   gtk_box_pack_start (GTK_BOX (main_vbox), com_frame, TRUE, TRUE, 0);
 
