@@ -45,10 +45,23 @@
  *                        Ghostview may hang when displaying the files.
  * V 1.07, PK, 14-Sep-99: Add resolution to image
  * V 1.08, PK, 16-Jan-2000: Add PostScript-Level 2 by Austin Donnelly
+ * V 1.09, PK, 15-Feb-2000: Force showpage on EPS-files
+ *                          Add "RunLength" compression
+ *                          Fix problem with "Level 2" toggle
+ * V 1.10, PK, 15-Mar-2000: For load EPSF, allow negative Bounding Box Values
+ *                          Save PS: dont start lines of image data with %%
+ *                          to prevent problems with stupid PostScript
+ *                          analyzer programs (Stanislav Brabec)
+ *                          Add BeginData/EndData comments
+ *                          Save PS: Set default rotation to 0
+ * V 1.11, PK, 20-Aug-2000: Fix problem with BoundingBox recognition
+ *                          for Mac files.
+ *                          Fix problem with loop when reading not all
+ *                          images of a multi page file.
  */
-#define VERSIO 1.08
-static char dversio[] = "v1.08  16-Jan-2000";
-static char ident[] = "@(#) GIMP PostScript/PDF file-plugin v1.08  16-Jan-2000";
+#define VERSIO 1.11
+static char dversio[] = "v1.11  20-Aug-2000";
+static char ident[] = "@(#) GIMP PostScript/PDF file-plugin v1.11  20-Aug-2000";
 
 #include "config.h"
 
@@ -64,6 +77,7 @@ static char ident[] = "@(#) GIMP PostScript/PDF file-plugin v1.08  16-Jan-2000";
 
 #include "libgimp/stdplugins-intl.h"
 
+
 #ifdef G_OS_WIN32
 #include <process.h>		/* For _getpid() */
 
@@ -131,7 +145,7 @@ static PSSaveVals psvals =
   5.0, 5.0,       /* Offset */
   1,              /* Unit is mm */
   1,              /* Keep edge ratio */
-  90,             /* Rotate */
+  0,              /* Rotate */
   2,              /* PostScript Level */
   0,              /* Encapsulated PostScript flag */
   0,              /* Preview flag */
@@ -196,8 +210,9 @@ static FILE  *ps_open       (gchar            *filename,
 			     const PSLoadVals *loadopt,
 			     gint             *llx,
 			     gint             *lly,
-			     int              *urx,
-			     gint             *ury);
+			     gint             *urx,
+			     gint             *ury,
+			     gint             *is_epsf);
 
 static void   ps_close      (FILE *ifp);
 
@@ -229,8 +244,6 @@ static void dither_grey     (guchar *grey,
 
 /* Dialog-handling */
 
-static void   init_gtk                  (void);
-
 static gint   load_dialog               (void);
 static void   load_ok_callback          (GtkWidget *widget,
 					 gpointer   data);
@@ -249,7 +262,6 @@ static void   save_ok_callback         (GtkWidget *widget,
 static void   save_unit_toggle_update  (GtkWidget *widget,
                                         gpointer   data);
 
-
 GPlugInInfo PLUG_IN_INFO =
 {
   NULL,  /* init_proc  */
@@ -262,6 +274,11 @@ GPlugInInfo PLUG_IN_INFO =
 /* The run mode */
 static GRunModeType l_run_mode;
 
+static void compress_packbits (int nin,
+                               unsigned char *src,
+                               int *nout,
+                               unsigned char *dst);
+
 
 static guint32 ascii85_buf;
 static int ascii85_len = 0;
@@ -280,6 +297,7 @@ ascii85_flush (FILE *ofp)
   char c[5];
   int i;
   gboolean zero_case = (ascii85_buf == 0);
+  static int max_linewidth = 75;
 
   for (i=4; i >= 0; i--)
     {
@@ -290,20 +308,26 @@ ascii85_flush (FILE *ofp)
    * at end of data. */
   if (zero_case && (ascii85_len == 4))
     {
+      if (ascii85_linewidth >= max_linewidth)
+      {
+        putc ('\n', ofp);
+        ascii85_linewidth = 0;
+      }
       putc ('z', ofp);
       ascii85_linewidth++;
     }
   else
     {
-      ascii85_linewidth += ascii85_len+1;
       for (i=0; i < ascii85_len+1; i++)
+      {
+        if ((ascii85_linewidth >= max_linewidth) && (c[i] != '%'))
+        {
+          putc ('\n', ofp);
+          ascii85_linewidth = 0;
+        }
 	putc (c[i], ofp);
-    }
-
-  if (ascii85_linewidth >= 75)
-    {
-      putc ('\n', ofp);
-      ascii85_linewidth = 0;
+        ascii85_linewidth++;
+      }
     }
 
   ascii85_len = 0;
@@ -321,6 +345,16 @@ ascii85_out (unsigned char byte, FILE *ofp)
   ascii85_len++;
 }
 
+static void
+ascii85_nout (int n, unsigned char *uptr, FILE *ofp)
+{
+ while (n-- > 0)
+ {
+   ascii85_out (*uptr, ofp);
+   uptr++;
+ }
+}
+
 static void
 ascii85_done (FILE *ofp)
 {
@@ -337,6 +371,133 @@ ascii85_done (FILE *ofp)
 }
 
 
+static void
+compress_packbits (int nin,
+                   unsigned char *src,
+                   int *nout,
+                   unsigned char *dst)
+
+{register unsigned char c;
+ int nrepeat, nliteral;
+ unsigned char *run_start;
+ unsigned char *start_dst = dst;
+ unsigned char *last_literal = NULL;
+
+ for (;;)
+ {
+   if (nin <= 0) break;
+
+   run_start = src;
+   c = *run_start;
+
+   /* Search repeat bytes */
+   if ((nin > 1) && (c == src[1]))
+   {
+     nrepeat = 1;
+     nin -= 2;
+     src += 2;
+     while ((nin > 0) && (c == *src))
+     {
+       nrepeat++;
+       src++;
+       nin--;
+       if (nrepeat == 127) break; /* Maximum repeat */
+     }
+
+     /* Add two-byte repeat to last literal run ? */
+     if (   (nrepeat == 1)
+         && (last_literal != NULL) && (((*last_literal)+1)+2 <= 128))
+     {
+       *last_literal += 2;
+       *(dst++) = c;
+       *(dst++) = c;
+       continue;
+     }
+
+     /* Add repeat run */
+     *(dst++) = (unsigned char)((-nrepeat) & 0xff);
+     *(dst++) = c;
+     last_literal = NULL;
+     continue;
+   }
+   /* Search literal bytes */
+   nliteral = 1;
+   nin--;
+   src++;
+
+   for (;;)
+   {
+     if (nin <= 0) break;
+
+     if ((nin >= 2) && (src[0] == src[1])) /* A two byte repeat ? */
+       break;
+
+     nliteral++;
+     nin--;
+     src++;
+     if (nliteral == 128) break; /* Maximum literal run */
+   }
+
+   /* Could be added to last literal run ? */
+   if ((last_literal != NULL) && (((*last_literal)+1)+nliteral <= 128))
+   {
+     *last_literal += nliteral;
+   }
+   else
+   {
+     last_literal = dst;
+     *(dst++) = (unsigned char)(nliteral-1);
+   }
+   while (nliteral-- > 0) *(dst++) = *(run_start++);
+ }
+ *nout = dst - start_dst;
+}
+
+
+typedef struct
+{
+  long eol;
+  long begin_data;
+} PS_DATA_POS;
+
+static PS_DATA_POS ps_data_pos = { 0, 0 };
+
+static void
+ps_begin_data (FILE *ofp)
+
+{
+                   /* %%BeginData: 123456789012 ASCII Bytes */
+ fprintf (ofp, "%s", "%%BeginData:                         ");
+ fflush (ofp);
+ ps_data_pos.eol = ftell (ofp);
+ fprintf (ofp, "\n");
+ fflush (ofp);
+ ps_data_pos.begin_data = ftell (ofp);
+}
+
+static void
+ps_end_data (FILE *ofp)
+
+{long end_data;
+ char s[64];
+
+ if ((ps_data_pos.begin_data > 0) && (ps_data_pos.eol > 0))
+ {
+   fflush (ofp);
+   end_data = ftell (ofp);
+   if (end_data > 0)
+   {
+     sprintf (s, "%ld ASCII Bytes", end_data-ps_data_pos.begin_data);
+     if (fseek (ofp, ps_data_pos.eol - strlen (s), SEEK_SET) == 0)
+     {
+       fprintf (ofp, "%s", s);
+       fseek (ofp, 0, SEEK_END);
+     }
+   }
+ }
+ fprintf (ofp, "%s\n", "%%EndData");
+}
+
 
 MAIN ()
 
@@ -392,8 +553,6 @@ query (void)
   };
   static gint nsave_args = sizeof (save_args) / sizeof (save_args[0]);
 
-  INIT_I18N();
-
   gimp_install_procedure ("file_ps_load",
                           "load file of PostScript/PDF file format",
                           "load file of PostScript/PDF file format",
@@ -421,7 +580,7 @@ query (void)
   gimp_install_procedure ("file_ps_save",
                           "save file in PostScript file format",
                           "PostScript saving handles all image types except those with alpha channels.",
-                          "Peter Kirchgessner <pkirchg@aol.com>",
+                          "Peter Kirchgessner <peter@kirchgessner.net>",
                           "Peter Kirchgessner",
                           dversio,
                           "<Save>/PostScript",
@@ -562,13 +721,13 @@ run (gchar   *name,
       image_ID = orig_image_ID = param[1].data.d_int32;
       drawable_ID = param[2].data.d_int32;
 
-      /* eventually export the image */ 
+      /* eventually export the image */
       switch (run_mode)
 	{
 	case RUN_INTERACTIVE:
 	case RUN_WITH_LAST_VALS:
-	  init_gtk ();
-	  export = gimp_export_image (&image_ID, &drawable_ID, "PS", 
+	  gimp_ui_init ("ps", FALSE);
+	  export = gimp_export_image (&image_ID, &drawable_ID, "PS",
 				      (CAN_HANDLE_RGB |
 				       CAN_HANDLE_GRAY |
 				       CAN_HANDLE_INDEXED));
@@ -697,6 +856,7 @@ load_image (gchar *filename)
   char *temp;
   int  llx, lly, urx, ury;
   int  k, n_images, max_images, max_pagenum;
+  int  is_epsf;
 
 #ifdef PS_DEBUG
   g_print ("load_image:\n resolution = %d\n", plvals.resolution);
@@ -723,7 +883,7 @@ load_image (gchar *filename)
       g_free (temp);
     }
 
-  ifp = ps_open (filename, &plvals, &llx, &lly, &urx, &ury);
+  ifp = ps_open (filename, &plvals, &llx, &lly, &urx, &ury, &is_epsf);
   if (!ifp)
     {
       g_message (_("PS: can't interprete file"));
@@ -735,6 +895,7 @@ load_image (gchar *filename)
   max_images = 10;
 
   max_pagenum = 9999;  /* Try to get the maximum pagenumber to read */
+  if (is_epsf) max_pagenum = 1;
   if (!page_in_list (plvals.pages, max_pagenum)) /* Is there a limit in list ? */
     {
       max_pagenum = -1;
@@ -757,7 +918,7 @@ load_image (gchar *filename)
 	  image_ID = load_ps (filename, page_count, ifp, llx, lly, urx, ury);
 	  if (image_ID == -1) break;
 #ifdef GIMP_HAVE_RESOLUTION_INFO
-	  gimp_image_set_resolution (image_ID, 
+	  gimp_image_set_resolution (image_ID,
 				     (double) plvals.resolution,
 				     (double) plvals.resolution);
 #endif
@@ -777,7 +938,6 @@ load_image (gchar *filename)
 	  if (image_ID == -1) break;
 	}
     }
-
   ps_close (ifp);
 
   /* Display images in reverse order. The last will be displayed by GIMP itself*/
@@ -1011,6 +1171,59 @@ page_in_list (gchar *list,
 }
 
 
+/* A function like fgets, but treats single CR-character as line break. */
+/* As a line break the newline-character is returned. */
+static char *psfgets (char *s, int size, FILE *stream)
+
+{int c;
+ char *sptr = s;
+
+ if (size <= 0) return NULL;
+ if (size == 1)
+ {
+   *s = '\0';
+   return NULL;
+ }
+ c = getc (stream);
+ if (c == EOF) return NULL;
+
+ for (;;)
+ {
+   /* At this point we have space in sptr for at least two characters */
+   if (c == '\n')    /* Got end of line (UNIX line end) ? */
+   {
+     *(sptr++) = '\n';
+     break;
+   }
+   else if (c == '\r')  /* Got a carriage return. Check next charcater */
+   {
+     c = getc (stream);
+     if ((c == EOF) || (c == '\n')) /* EOF or DOS line end ? */
+     {
+       *(sptr++) = '\n';  /* Return UNIX line end */
+       break;
+     }
+     else  /* Single carriage return. Return UNIX line end. */
+     {
+       ungetc (c, stream);  /* Save the extra character */
+       *(sptr++) = '\n';
+       break;
+     }
+   }
+   else   /* no line end character */
+   {
+     *(sptr++) = (char)c;
+     size--;
+   }
+   if (size == 1) break;  /* Only space for the nul-character ? */
+   c = getc (stream);
+   if (c == EOF) break;
+ }
+ *sptr = '\0';
+ return s;
+}
+
+
 /* Get the BoundingBox of a PostScript file. On success, 0 is returned. */
 /* On failure, -1 is returned. */
 static gint
@@ -1029,7 +1242,7 @@ get_bbox (gchar *filename,
 
   for (;;)
     {
-      if (fgets (line, sizeof (line)-1, ifp) == NULL) break;
+      if (psfgets (line, sizeof (line)-1, ifp) == NULL) break;
       if ((line[0] != '%') || (line[1] != '%')) continue;
       src = &(line[2]);
       while ((*src == ' ') || (*src == '\t')) src++;
@@ -1059,14 +1272,17 @@ ps_open (gchar            *filename,
          gint             *llx,
          gint             *lly,
          gint             *urx,
-         gint             *ury)
+         gint             *ury,
+         gint             *is_epsf)
 {
   char *cmd, *gs, *gs_opts, *driver;
   FILE *fd_popen;
   int width, height, resolution;
   int x0, y0, x1, y1;
+  int offx = 0, offy = 0;
   int is_pdf;
   char TextAlphaBits[64], GraphicsAlphaBits[64], geometry[32];
+  char offset[32];
 
   resolution = loadopt->resolution;
   *llx = *lly = 0;
@@ -1077,6 +1293,8 @@ ps_open (gchar            *filename,
 
   /* Check if the file is a PDF. For PDF, we cant set geometry */
   is_pdf = 0;
+  /* Check if it is a EPS-file */
+  *is_epsf = 0;
 #ifndef __EMX__
   fd_popen = fopen (filename, "r");
 #else
@@ -1084,25 +1302,44 @@ ps_open (gchar            *filename,
 #endif
   if (fd_popen != NULL)
     {
-      char hdr[4];
+      char hdr[512];
 
-      fread (hdr, 1, 4, fd_popen);
+      fread (hdr, 1, sizeof(hdr), fd_popen);
       is_pdf = (strncmp (hdr, "%PDF", 4) == 0);
+
+      if (!is_pdf)  /* Check for EPSF */
+      {char *adobe, *epsf;
+       int ds = 0;
+
+        hdr[sizeof(hdr)-1] = '\0';
+        adobe = strstr (hdr, "PS-Adobe-");
+        epsf = strstr (hdr, "EPSF-");
+        if ((adobe != NULL) && (epsf != NULL))
+          ds = epsf - adobe;
+        *is_epsf = ((ds >= 11) && (ds <= 15));
+      }
       fclose (fd_popen);
     }
 
   if ((!is_pdf) && (loadopt->use_bbox))    /* Try the BoundingBox ? */
     {
-      if (   (get_bbox (filename, &x0, &y0, &x1, &y1) == 0)
-	     && (x0 >= 0) && (y0 >= 0) && (x1 > x0) && (y1 > y0))
-	{
-	  *llx = (int)((x0/72.0) * resolution + 0.01);
-	  *lly = (int)((y0/72.0) * resolution + 0.01);
-	  *urx = (int)((x1/72.0) * resolution + 0.01);
-	  *ury = (int)((y1/72.0) * resolution + 0.01);
-	  width = *urx + 1;
-	  height = *ury + 1;
-	}
+      if (get_bbox (filename, &x0, &y0, &x1, &y1) == 0)
+        {
+          if (*is_epsf)  /* Handle negative BoundingBox for EPSF */
+            {
+              offx = -x0; x1 += offx; x0 += offx;
+              offy = -y0; y1 += offy; y0 += offy;
+            }
+          if ((x0 >= 0) && (y0 >= 0) && (x1 > x0) && (y1 > y0))
+            {
+               *llx = (int)((x0/72.0) * resolution + 0.01);
+               *lly = (int)((y0/72.0) * resolution + 0.01);
+               *urx = (int)((x1/72.0) * resolution + 0.01);
+               *ury = (int)((y1/72.0) * resolution + 0.01);
+               width = *urx + 1;
+               height = *ury + 1;
+            }
+        }
     }
   if (loadopt->pnm_type == 4) driver = "pbmraw";
   else if (loadopt->pnm_type == 5) driver = "pgmraw";
@@ -1140,6 +1377,11 @@ ps_open (gchar            *filename,
     gs_opts = "";  /* Ghostscript will add these options */
 
   TextAlphaBits[0] = GraphicsAlphaBits[0] = geometry[0] = '\0';
+  offset[0] = '\0';
+
+  /* Offset command for gs to get image part with negative x/y-coord. */
+  if ((offx != 0) || (offy != 0))
+    sprintf (offset, "-c %d %d translate -- ", offx, offy);
 
   /* Antialiasing not available for PBM-device */
   if ((loadopt->pnm_type != 4) && (loadopt->textalpha != 1))
@@ -1153,10 +1395,11 @@ ps_open (gchar            *filename,
     sprintf (geometry,"-g%dx%d ", width, height);
 
   cmd = g_strdup_printf ("%s -sDEVICE=%s -r%d %s%s%s-q -dNOPAUSE %s \
--sOutputFile=%s %s -c quit",
+-sOutputFile=%s %s%s %s-c quit",
 			 gs, driver, resolution, geometry,
 			 TextAlphaBits, GraphicsAlphaBits,
-			 gs_opts, pnmfile, filename);
+			 gs_opts, pnmfile, offset, filename,
+                         *is_epsf ? "-c showpage " : "");
 #ifdef PS_DEBUG
   g_print ("Going to start ghostscript with:\n%s\n", cmd);
 #endif
@@ -1206,6 +1449,11 @@ static void
 ps_close (FILE *ifp)
 {
 #ifndef USE_REAL_OUTPUTFILE
+
+  /* Even if we dont want all images, we have to read the pipe until EOF. */
+  /* Otherwise the subprocess and therefore pclose() may not finish. */
+  while (getc (ifp) != EOF) ;
+
   /* Finish reading from pipe. */
   pclose (ifp);
 #else
@@ -1579,7 +1827,7 @@ save_ps_setup (FILE   *ofp,
 
   fprintf (ofp, "%%%%BeginProlog\n");
   fprintf (ofp, "%% Use own dictionary to avoid conflicts\n");
-  fprintf (ofp, "5 dict begin\n");
+  fprintf (ofp, "10 dict begin\n");
   fprintf (ofp, "%%%%EndProlog\n");
   fprintf (ofp, "%%%%Page: 1 1\n");
   fprintf (ofp, "%% Translate for offset\n");
@@ -1843,6 +2091,7 @@ save_gray  (FILE   *ofp,
   int height, width, i, j;
   int tile_height;
   unsigned char *data, *src;
+  unsigned char *packb = NULL;
   GPixelRgn pixel_rgn;
   GDrawable *drawable;
   GDrawableType drawable_type;
@@ -1869,9 +2118,12 @@ save_gray  (FILE   *ofp,
   }
   else
   {
-    fprintf (ofp, "currentfile /ASCII85Decode filter\n");
+    fprintf (ofp,"currentfile /ASCII85Decode filter /RunLengthDecode filter\n");
     ascii85_init ();
+    /* Allocate buffer for packbits data. Worst case: Less than 1% increase */
+    packb = (guchar *)g_malloc ((width * 105)/100+2);
   }
+  ps_begin_data (ofp);
   fprintf (ofp, "image\n");
 
 #define GET_GRAY_TILE(begin) \
@@ -1894,16 +2146,24 @@ save_gray  (FILE   *ofp,
 	  putc ('\n', ofp);
 	}
       else
-	{
-	  for (j = 0; j < width; j++)
-	    ascii85_out (*(src++), ofp);
+	{int nout;
+
+          compress_packbits (width, src, &nout, packb);
+          ascii85_nout (nout, packb, ofp);
+          src += width;
 	}
       if ((l_run_mode != RUN_NONINTERACTIVE) && ((i % 20) == 0))
 	gimp_progress_update ((double) i / (double) height);
     }
-  if (level2) ascii85_done (ofp);
+  if (level2)
+  {
+    ascii85_out (128, ofp); /* Write EOD of RunLengthDecode filter */
+    ascii85_done (ofp);
+  }
+  ps_end_data (ofp);
   fprintf (ofp, "showpage\n");
   g_free (data);
+  if (packb) g_free (packb);
 
   gimp_drawable_detach (drawable);
 
@@ -1927,6 +2187,7 @@ save_bw (FILE   *ofp,
   int tile_height;
   guchar *cmap, *ct;
   guchar *data, *src;
+  guchar *packb = NULL;
   guchar *scanline, *dst, mask;
   guchar *hex_scanline;
   GPixelRgn pixel_rgn;
@@ -1960,9 +2221,12 @@ save_bw (FILE   *ofp,
   }
   else
   {
-    fprintf (ofp, "currentfile /ASCII85Decode filter\n");
+    fprintf (ofp,"currentfile /ASCII85Decode filter /RunLengthDecode filter\n");
     ascii85_init ();
+    /* Allocate buffer for packbits data. Worst case: Less than 1% increase */
+    packb = (guchar *)g_malloc (((nbsl+1) * 105)/100+2);
   }
+  ps_begin_data (ofp);
   fprintf (ofp, "image\n");
 
 #define GET_BW_TILE(begin) \
@@ -2006,20 +2270,26 @@ save_bw (FILE   *ofp,
 	    }
 	}
       else
-	{
-	  dst = scanline;
-	  for (j = 0; j < nbsl; j++)
-	    ascii85_out (*(dst++), ofp);
+	{int nout;
+
+          compress_packbits (nbsl, scanline, &nout, packb);
+          ascii85_nout (nout, packb, ofp);
 	}
       if ((l_run_mode != RUN_NONINTERACTIVE) && ((i % 20) == 0))
 	gimp_progress_update ((double) i / (double) height);
     }
-  if (level2) ascii85_done (ofp);
+  if (level2)
+  {
+    ascii85_out (128, ofp); /* Write EOD of RunLengthDecode filter */
+    ascii85_done (ofp);
+  }
+  ps_end_data (ofp);
   fprintf (ofp, "showpage\n");
 
   g_free (hex_scanline);
   g_free (scanline);
   g_free (data);
+  if (packb != NULL) g_free (packb);
 
   gimp_drawable_detach (drawable);
 
@@ -2041,8 +2311,9 @@ save_index (FILE   *ofp,
   int height, width, i, j;
   int ncols, bw;
   int tile_height;
-  unsigned char *cmap, *cmap_start;
-  unsigned char *data, *src;
+  guchar *cmap, *cmap_start;
+  guchar *data, *src;
+  guchar *packb = NULL, *plane = NULL;
   char coltab[256*6], *ct;
   GPixelRgn pixel_rgn;
   GDrawable *drawable;
@@ -2096,9 +2367,26 @@ save_index (FILE   *ofp,
   }
   else
   {
-    fprintf (ofp, "currentfile /ASCII85Decode filter false 3\n");
-    ascii85_init ();
+    fprintf (ofp, "%% Strings to hold RGB-samples per scanline\n");
+    fprintf (ofp, "/rstr %d string def\n", width);
+    fprintf (ofp, "/gstr %d string def\n", width);
+    fprintf (ofp, "/bstr %d string def\n", width);
+    fprintf (ofp,
+            "{currentfile /ASCII85Decode filter /RunLengthDecode filter\
+ rstr readstring pop}\n");
+    fprintf (ofp,
+            "{currentfile /ASCII85Decode filter /RunLengthDecode filter\
+ gstr readstring pop}\n");
+    fprintf (ofp,
+            "{currentfile /ASCII85Decode filter /RunLengthDecode filter\
+ bstr readstring pop}\n");
+    fprintf (ofp, "true 3\n");
+
+    /* Allocate buffer for packbits data. Worst case: Less than 1% increase */
+    packb = (guchar *)g_malloc ((width * 105)/100+2);
+    plane = (guchar *)g_malloc (width);
   }
+  ps_begin_data (ofp);
   fprintf (ofp, "colorimage\n");
 
 #define GET_INDEX_TILE(begin) \
@@ -2120,22 +2408,32 @@ save_index (FILE   *ofp,
 	  putc ('\n', ofp);
 	}
       else
-	{
-	  for (j = 0; j < width; j++)
-	    {
-	      cmap = cmap_start + 3 * (*(src++));
-	      ascii85_out (*(cmap++), ofp);
-	      ascii85_out (*(cmap++), ofp);
-	      ascii85_out (*(cmap++), ofp);
-	    }
-	}
+        {guchar *plane_ptr, *src_ptr;
+         int rgb, nout;
+
+          for (rgb = 0; rgb < 3; rgb++)
+          {
+            src_ptr = src;
+            plane_ptr = plane;
+            for (j = 0; j < width; j++)
+              *(plane_ptr++) = cmap_start[3 * *(src_ptr++) + rgb];
+            compress_packbits (width, plane, &nout, packb);
+            ascii85_init ();
+            ascii85_nout (nout, packb, ofp);
+            ascii85_out (128, ofp); /* Write EOD of RunLengthDecode filter */
+            ascii85_done (ofp);
+          }
+          src += width;
+        }
       if ((l_run_mode != RUN_NONINTERACTIVE) && ((i % 20) == 0))
 	gimp_progress_update ((double) i / (double) height);
     }
-  if (level2) ascii85_done (ofp);
+  ps_end_data (ofp);
   fprintf (ofp, "showpage\n");
 
   g_free (data);
+  if (packb != NULL) g_free (packb);
+  if (plane != NULL) g_free (plane);
 
   gimp_drawable_detach (drawable);
 
@@ -2157,6 +2455,7 @@ save_rgb (FILE   *ofp,
   int height, width, tile_height;
   int i, j;
   guchar *data, *src;
+  guchar *packb = NULL, *plane = NULL;
   GPixelRgn pixel_rgn;
   GDrawable *drawable;
   GDrawableType drawable_type;
@@ -2183,9 +2482,26 @@ save_rgb (FILE   *ofp,
   }
   else
   {
-    fprintf (ofp, "currentfile /ASCII85Decode filter false 3\n");
-    ascii85_init ();
+    fprintf (ofp, "%% Strings to hold RGB-samples per scanline\n");
+    fprintf (ofp, "/rstr %d string def\n", width);
+    fprintf (ofp, "/gstr %d string def\n", width);
+    fprintf (ofp, "/bstr %d string def\n", width);
+    fprintf (ofp,
+            "{currentfile /ASCII85Decode filter /RunLengthDecode filter\
+ rstr readstring pop}\n");
+    fprintf (ofp,
+            "{currentfile /ASCII85Decode filter /RunLengthDecode filter\
+ gstr readstring pop}\n");
+    fprintf (ofp,
+            "{currentfile /ASCII85Decode filter /RunLengthDecode filter\
+ bstr readstring pop}\n");
+    fprintf (ofp, "true 3\n");
+
+    /* Allocate buffer for packbits data. Worst case: Less than 1% increase */
+    packb = (guchar *)g_malloc ((width * 105)/100+2);
+    plane = (guchar *)g_malloc (width);
   }
+  ps_begin_data (ofp);
   fprintf (ofp, "colorimage\n");
 
 #define GET_RGB_TILE(begin) \
@@ -2212,20 +2528,34 @@ save_rgb (FILE   *ofp,
 	  putc ('\n', ofp);
 	}
       else
-	{
-	  for (j = 0; j < width; j++)
+	{guchar *plane_ptr, *src_ptr;
+         int rgb, nout;
+
+          for (rgb = 0; rgb < 3; rgb++)
+          {
+            src_ptr = src + rgb;
+            plane_ptr = plane;
+            for (j = 0; j < width; j++)
 	    {
-	      ascii85_out (*(src++), ofp);
-	      ascii85_out (*(src++), ofp);
-	      ascii85_out (*(src++), ofp);
-	    }
+              *(plane_ptr++) = *src_ptr;
+              src_ptr += 3;
+            }
+            compress_packbits (width, plane, &nout, packb);
+            ascii85_init ();
+	    ascii85_nout (nout, packb, ofp);
+            ascii85_out (128, ofp); /* Write EOD of RunLengthDecode filter */
+            ascii85_done (ofp);
+          }
+          src += 3*width;
 	}
       if ((l_run_mode != RUN_NONINTERACTIVE) && ((i % 20) == 0))
 	gimp_progress_update ((double) i / (double) height);
     }
-  if (level2) ascii85_done (ofp);
+  ps_end_data (ofp);
   fprintf (ofp, "showpage\n");
   g_free (data);
+  if (packb != NULL) g_free (packb);
+  if (plane != NULL) g_free (plane);
 
   gimp_drawable_detach (drawable);
 
@@ -2238,20 +2568,6 @@ save_rgb (FILE   *ofp,
 #undef GET_RGB_TILE
 }
 
-static void 
-init_gtk (void)
-{
-  gchar **argv;
-  gint    argc;
-
-  argc    = 1;
-  argv    = g_new (gchar *, 1);
-  argv[0] = g_strdup ("ps");
-
-  gtk_init (&argc, &argv);
-  gtk_rc_parse (gimp_gtkrc ());
-}
-
 /*  Load interface functions  */
 
 static gint
@@ -2268,10 +2584,10 @@ load_dialog (void)
   GtkWidget *pages_entry;
   GtkWidget *toggle;
 
-  init_gtk ();
+  gimp_ui_init ("ps", FALSE);
 
   dialog = gimp_dialog_new (_("Load PostScript"), "ps",
-			    gimp_plugin_help_func, "filters/ps.html",
+			    gimp_standard_help_func, "filters/ps.html",
 			    GTK_WIN_POS_MOUSE,
 			    FALSE, TRUE, FALSE,
 
@@ -2447,12 +2763,13 @@ save_dialog (void)
   GtkObject *adj;
   gint       j;
 
-  vals = g_new (SaveDialogVals, 1);
+  gimp_help_init ();
 
+  vals = g_new (SaveDialogVals, 1);
   vals->level = (psvals.level > 1);
 
   dialog = gimp_dialog_new (_("Save as PostScript"), "ps",
-			    gimp_plugin_help_func, "filters/ps.html",
+			    gimp_standard_help_func, "filters/ps.html",
 			    GTK_WIN_POS_MOUSE,
 			    FALSE, TRUE, FALSE,
 
@@ -2483,7 +2800,7 @@ save_dialog (void)
   /* Image Size */
   frame = gtk_frame_new (_("Image Size"));
   gtk_frame_set_shadow_type (GTK_FRAME (frame), GTK_SHADOW_ETCHED_IN);
-  gtk_box_pack_start (GTK_BOX (main_vbox[0]), frame, FALSE, FALSE, 0);
+  gtk_box_pack_start (GTK_BOX (main_vbox[0]), frame, TRUE, TRUE, 0);
 
   vbox = gtk_vbox_new (FALSE, 4);
   gtk_container_set_border_width (GTK_CONTAINER (vbox), 4);
@@ -2533,6 +2850,9 @@ save_dialog (void)
 		      &psvals.y_offset);
 
   toggle = gtk_check_button_new_with_label (_("Keep Aspect Ratio"));
+  gimp_help_set_help_data (toggle, _("When toggled, the resulting image will be scaled to fit "
+				     "into the given size without changing the aspect ratio."),
+			   "#keep_aspect_ratio"), 
   gtk_box_pack_start (GTK_BOX (vbox), toggle, TRUE, TRUE, 0);
   gtk_signal_connect (GTK_OBJECT (toggle), "toggled",
                       GTK_SIGNAL_FUNC (gimp_toggle_button_update),
@@ -2633,7 +2953,7 @@ save_dialog (void)
   gtk_main ();
   gdk_flush ();
 
-  psvals.level = (vals->level < 1);
+  psvals.level = (vals->level) ? 2 : 1;
 
   g_free (vals);
 
