@@ -23,8 +23,9 @@
  * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
  * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
- * $Id: waves.c,v 1.12 2000/02/04 15:12:12 mitch Exp $
+ * $Id: waves.c,v 1.15 2000/08/20 20:34:26 austin Exp $
  */
+
 #include "config.h"
 
 #include <string.h>
@@ -37,6 +38,7 @@
 
 #include "libgimp/stdplugins-intl.h"
 
+
 enum
 {
   MODE_SMEAR,
@@ -123,10 +125,10 @@ static guchar bilinear (gdouble x,
 
 GPlugInInfo PLUG_IN_INFO =
 {
-  NULL,  /* init  */
-  NULL,  /* quit  */
-  query, /* query */
-  run,   /* run   */
+  NULL,  /* init_proc  */
+  NULL,  /* quit_proc  */
+  query, /* query_proc */
+  run,   /* run_proc   */
 };
 
 MAIN ()
@@ -143,12 +145,10 @@ query (void)
     { PARAM_FLOAT, "phase", "The Phase of the Waves" },
     { PARAM_FLOAT, "wavelength", "The Wavelength of the Waves" },
     { PARAM_INT32, "type", "Type of waves, black/smeared" },
-    { PARAM_INT32, "reflective", "Use Reflection" },
+    { PARAM_INT32, "reflective", "Use Reflection" }
   };
   static gint nargs = sizeof (args) / sizeof (args[0]);
 
-  INIT_I18N();
-
   gimp_install_procedure ("plug_in_waves",
 			  "Distort the image with waves",
 			  "none yet",
@@ -309,7 +309,7 @@ waves_radio_button_update (GtkWidget *widget,
 {
   gimp_radio_button_update (widget, data);
 
-  if (do_preview)
+  if (do_preview && GTK_TOGGLE_BUTTON (widget)->active)
     waves_do_preview (NULL);
 }
 
@@ -338,18 +338,11 @@ pluginCoreIA (struct piArgs *argp,
   GtkWidget *preview;
   GtkWidget *toggle;
   GtkObject *adj;
-  gchar **argv;
-  gint    argc;
 
-  argc    = 1;
-  argv    = g_new (gchar *, 1);
-  argv[0] = g_strdup ("waves");
-
-  gtk_init (&argc, &argv);
-  gtk_rc_parse (gimp_gtkrc ());
+  gimp_ui_init ("waves", TRUE);
 
   dlg = gimp_dialog_new ( _("Waves"), "waves",
-			 gimp_plugin_help_func, "filters/waves.html",
+			 gimp_standard_help_func, "filters/waves.html",
 			 GTK_WIN_POS_MOUSE,
 			 FALSE, TRUE, FALSE,
 
@@ -579,16 +572,6 @@ mw_preview_new (GtkWidget        *parent,
   GtkWidget *pframe;
   GtkWidget *vbox;
   GtkWidget *button;
-  guchar *color_cube;
-   
-  gtk_preview_set_gamma (gimp_gamma ());
-  gtk_preview_set_install_cmap (gimp_install_cmap ());
-  color_cube = gimp_color_cube ();
-  gtk_preview_set_color_cube (color_cube[0], color_cube[1],
-                              color_cube[2], color_cube[3]);
-
-  gtk_widget_set_default_visual (gtk_preview_get_visual ());
-  gtk_widget_set_default_colormap (gtk_preview_get_cmap ());
 
   frame = gtk_frame_new (_("Preview"));
   gtk_frame_set_shadow_type (GTK_FRAME (frame), GTK_SHADOW_ETCHED_IN);
@@ -622,8 +605,6 @@ mw_preview_new (GtkWidget        *parent,
 }
 
 
-
-
 static void
 wave (guchar  *src,
       guchar  *dst,
