@@ -31,7 +31,6 @@
  *   - Parsing is very tolerant, and the algorithms are quite hairy, so
  *     load_image should be carefully tested to make sure there are no XBM's
  *     that fail.
- *   - Allow the user to specify a hotspot, and preserve it when loading.
  */
 
 /* Set this for debugging. */
@@ -51,9 +50,11 @@
 
 #include "libgimp/stdplugins-intl.h"
 
+
 /* Wear your GIMP with pride! */
 #define DEFAULT_USE_COMMENT TRUE
 #define MAX_COMMENT         72
+#define MAX_MASK_EXT        32
 
 /* C identifier prefix. */
 #define DEFAULT_PREFIX "bitmap"
@@ -64,20 +65,26 @@
 
 typedef struct _XBMSaveVals
 {
-  gchar comment[MAX_COMMENT + 1];
-  gint  x10_format;
-  gint  x_hot;
-  gint  y_hot;
-  gchar prefix[MAX_PREFIX + 1];
+  gchar    comment[MAX_COMMENT + 1];
+  gint     x10_format;
+  gint     use_hot;
+  gint     x_hot;
+  gint     y_hot;
+  gchar    prefix[MAX_PREFIX + 1];
+  gboolean write_mask;
+  gchar    mask_ext[MAX_MASK_EXT + 1];
 } XBMSaveVals;
 
 static XBMSaveVals xsvals =
 {
   "###",		/* comment */
   DEFAULT_X10_FORMAT,	/* x10_format */
-  -1,			/* x_hot */
-  -1,			/* y_hot */
+  FALSE,
+  0,			/* x_hot */
+  0,			/* y_hot */
   DEFAULT_PREFIX,	/* prefix */
+  FALSE,                /* write_mask */
+  "_mask"
 };
 
 typedef struct _XBMSaveInterface
@@ -94,26 +101,30 @@ static XBMSaveInterface xsint =
 /* Declare some local functions.
  */
 static void   query   (void);
-static void   run     (gchar   *name,
-		       gint     nparams,
-		       GParam  *param,
-		       gint    *nreturn_vals,
-		       GParam **return_vals);
+static void   run     (gchar      *name,
+		       gint        nparams,
+		       GimpParam  *param,
+		       gint       *nreturn_vals,
+		       GimpParam **return_vals);
 
-static void   init_gtk               (void);
-static gint32 load_image             (gchar     *filename);
-static gint   save_image             (gchar     *filename,
-				      gint32     image_ID, 
-				      gint32     drawable_ID);
-static gint   save_dialog            (gint32     drawable_ID);
-static void   save_ok_callback       (GtkWidget *widget,
-				      gpointer   data);
-static void   comment_entry_callback (GtkWidget *widget,
-				      gpointer   data);
-static void   prefix_entry_callback  (GtkWidget *widget,
-				      gpointer   data);
+static gint32 load_image              (gchar     *filename);
+static gint   save_image              (gchar     *filename,
+				       gchar     *prefix,
+				       gchar     *comment,
+				       gboolean   save_mask,
+				       gint32     image_ID, 
+				       gint32     drawable_ID);
+static gint   save_dialog             (gint32     drawable_ID);
+static void   save_ok_callback        (GtkWidget *widget,
+				       gpointer   data);
+static void   comment_entry_callback  (GtkWidget *widget,
+				       gpointer   data);
+static void   prefix_entry_callback   (GtkWidget *widget,
+				       gpointer   data);
+static void   mask_ext_entry_callback (GtkWidget *widget,
+				       gpointer   data);
 
-GPlugInInfo PLUG_IN_INFO =
+GimpPlugInInfo PLUG_IN_INFO =
 {
   NULL,  /* init_proc  */
   NULL,  /* quit_proc  */
@@ -130,37 +141,38 @@ static int verbose = VERBOSE;
 static void
 query (void)
 {
-  static GParamDef load_args[] =
+  static GimpParamDef load_args[] =
   {
     { PARAM_INT32,  "run_mode",     "Interactive, non-interactive" },
     { PARAM_STRING, "filename",     "The name of the file to load" },
-    { PARAM_STRING, "raw_filename", "The name entered" },
-  };
-  static GParamDef load_return_vals[] =
-  {
-    { PARAM_IMAGE,  "image",        "Output image" },
+    { PARAM_STRING, "raw_filename", "The name entered" }
   };
   static gint nload_args = sizeof (load_args) / sizeof (load_args[0]);
+
+  static GimpParamDef load_return_vals[] =
+  {
+    { PARAM_IMAGE,  "image",        "Output image" }
+  };
   static gint nload_return_vals = (sizeof (load_return_vals) /
 				   sizeof (load_return_vals[0]));
 
-  static GParamDef save_args[] =
+  static GimpParamDef save_args[] =
   {
-    { PARAM_INT32,    "run_mode",     "Interactive, non-interactive" },
-    { PARAM_IMAGE,    "image",        "Input image" },
-    { PARAM_DRAWABLE, "drawable",     "Drawable to save" },
-    { PARAM_STRING,   "filename",     "The name of the file to save" },
-    { PARAM_STRING,   "raw_filename", "The name entered" },
-    { PARAM_STRING,   "comment",      "Image description (maximum 72 bytes)" },
-    { PARAM_INT32,    "x10",          "Save in X10 format" },
-    { PARAM_INT32,    "x_hot",        "X coordinate of hotspot" },
-    { PARAM_INT32,    "y_hot",        "Y coordinate of hotspot" },
-    { PARAM_STRING,   "prefix",       "Identifier prefix [determined from filename]"},
+    { PARAM_INT32,    "run_mode",       "Interactive, non-interactive" },
+    { PARAM_IMAGE,    "image",          "Input image" },
+    { PARAM_DRAWABLE, "drawable",       "Drawable to save" },
+    { PARAM_STRING,   "filename",       "The name of the file to save" },
+    { PARAM_STRING,   "raw_filename",   "The name entered" },
+    { PARAM_STRING,   "comment",        "Image description (maximum 72 bytes)" },
+    { PARAM_INT32,    "x10",            "Save in X10 format" },
+    { PARAM_INT32,    "x_hot",          "X coordinate of hotspot" },
+    { PARAM_INT32,    "y_hot",          "Y coordinate of hotspot" },
+    { PARAM_STRING,   "prefix",         "Identifier prefix [determined from filename]"},
+    { PARAM_INT32,    "write_mask",     "(0 = ignore, 1 = save as extra file)" },
+    { PARAM_STRING,   "mask_extension", "Extension of the mask file" }
   } ;
   static gint nsave_args = sizeof (save_args) / sizeof (save_args[0]);
 
-  INIT_I18N ();
-
   gimp_install_procedure ("file_xbm_load",
                           "Load a file in X10 or X11 bitmap (XBM) file format",
                           "Load a file in X10 or X11 bitmap (XBM) file format.  XBM is a lossless format for flat black-and-white (two color indexed) images.",
@@ -193,18 +205,13 @@ query (void)
 			      "");
 }
 
-static void
+static gchar *
 init_prefix (gchar *filename)
 {
   gchar *p, *prefix;
   gint len;
 
-  /* Mangle the filename to get the prefix. */
-  prefix = strrchr (filename, '/');
-  if (prefix)
-    prefix ++;
-  else
-    prefix = filename;
+  prefix = g_basename (filename);
 
   /* Strip any extension. */
   p = strrchr (prefix, '.');
@@ -215,20 +222,24 @@ init_prefix (gchar *filename)
 
   memset (xsvals.prefix, 0, sizeof (xsvals.prefix));
   strncpy (xsvals.prefix, prefix, len);
+
+  return xsvals.prefix;
 }
 
 static void
-run (gchar   *name,
-     gint     nparams,
-     GParam  *param,
-     gint    *nreturn_vals,
-     GParam **return_vals)
+run (gchar      *name,
+     gint        nparams,
+     GimpParam  *param,
+     gint       *nreturn_vals,
+     GimpParam **return_vals)
 {
-  static GParam values[2];
-  GRunModeType  run_mode;
-  GStatusType   status = STATUS_SUCCESS;
-  gint32        image_ID;
-  gint32        drawable_ID;
+  static GimpParam   values[2];
+  GimpRunModeType    run_mode;
+  GimpPDBStatusType  status = STATUS_SUCCESS;
+  gint32             image_ID;
+  gint32             drawable_ID;
+  GimpParasite      *parasite;
+  gchar             *mask_filename = NULL;
   GimpExportReturnType export = EXPORT_CANCEL;
 
   INIT_I18N_UI();
@@ -265,15 +276,16 @@ run (gchar   *name,
     {
       image_ID = param[1].data.d_int32;
       drawable_ID = param[2].data.d_int32;
-    
+
       /*  eventually export the image */ 
       switch (run_mode)
 	{
 	case RUN_INTERACTIVE:
 	case RUN_WITH_LAST_VALS:
-	  init_gtk ();
+	  gimp_ui_init ("xbm", FALSE);
 	  export = gimp_export_image (&image_ID, &drawable_ID, "XBM",
-				      CAN_HANDLE_INDEXED);
+				      CAN_HANDLE_INDEXED |
+				      CAN_HANDLE_ALPHA);
 	  if (export == EXPORT_CANCEL)
 	    {
 	      values[0].data.d_status = STATUS_CANCEL;
@@ -287,15 +299,12 @@ run (gchar   *name,
       switch (run_mode)
 	{
 	case RUN_INTERACTIVE:
+	case RUN_WITH_LAST_VALS:
 	  /*  Possibly retrieve data  */
 	  gimp_get_data ("file_xbm_save", &xsvals);
 
 	  /* Always override the prefix with the filename. */
-	  init_prefix (param[3].data.d_string);
-
-	  /*  First acquire information with a dialog  */
-	  if (! save_dialog (drawable_ID))
-	    status = STATUS_CANCEL;
+	  mask_filename = g_strdup (init_prefix (param[3].data.d_string));
 	  break;
 
 	case RUN_NONINTERACTIVE:
@@ -323,10 +332,13 @@ run (gchar   *name,
 	      if (nparams > i)
 		{
 		  /* They've asked for a hotspot. */
+		  xsvals.use_hot = TRUE;
 		  xsvals.x_hot = param[i - 1].data.d_int32;
 		  xsvals.y_hot = param[i].data.d_int32;
 		}
 
+	      mask_filename = g_strdup (init_prefix (param[3].data.d_string));
+
 	      i ++;
 	      if (nparams > i)
 		{
@@ -334,8 +346,15 @@ run (gchar   *name,
 		  strncpy (xsvals.prefix, param[i].data.d_string,
 			   MAX_PREFIX);
 		}
-	      else
-		init_prefix (param[3].data.d_string);
+
+	      i += 2;
+	      if (nparams > i)
+		{
+		  xsvals.write_mask = param[i - 1].data.d_int32;
+		  memset (xsvals.mask_ext, 0, sizeof (xsvals.mask_ext));
+		  strncpy (xsvals.mask_ext, param[i].data.d_string,
+			   MAX_MASK_EXT);
+		}
 
 	      i ++;
 	      /* Too many arguments. */
@@ -344,18 +363,99 @@ run (gchar   *name,
 	    }
 	  break;
 
-	case RUN_WITH_LAST_VALS:
-	  /*  Possibly retrieve data  */
-	  gimp_get_data ("file_xbm_save", &xsvals);
-	  break;
-
 	default:
 	  break;
 	}
 
+      /* Get the parasites */
+      parasite = gimp_image_parasite_find (image_ID, "gimp-comment");
+
+      if (parasite)
+	{
+	  gpointer data;
+	  gint     size;
+
+	  data = gimp_parasite_data (parasite);
+	  size = gimp_parasite_data_size (parasite);
+
+	  strncpy (xsvals.comment, data, MIN (size, MAX_COMMENT));
+	  xsvals.comment[MIN (size, MAX_COMMENT) + 1] = 0;
+
+	  gimp_parasite_free (parasite);
+	}
+
+      parasite = gimp_image_parasite_find (image_ID, "hot-spot");
+
+      if (parasite)
+	{
+	  gpointer data;
+	  gint     x, y;
+
+	  data = gimp_parasite_data (parasite);
+
+	  if (sscanf (data, "%i %i", &x, &y) == 2)
+	    {
+	      xsvals.use_hot = TRUE;
+	      xsvals.x_hot = x;
+	      xsvals.y_hot = y;
+	    }
+
+	  gimp_parasite_free (parasite);
+	}
+
+      if (run_mode == RUN_INTERACTIVE)
+	{
+	  /*  Acquire information with a dialog  */
+	  if (! save_dialog (drawable_ID))
+	    status = STATUS_CANCEL;
+	}
+
       if (status == STATUS_SUCCESS)
 	{
-	  if (save_image (param[3].data.d_string, image_ID, drawable_ID))
+	  gchar *temp;
+	  gchar *mask_prefix;
+	  gchar *dirname;
+
+	  temp = mask_filename;
+
+	  if ((dirname = g_dirname (param[3].data.d_string)) != NULL)
+	    {
+	      mask_filename = g_strdup_printf ("%s/%s%s.xbm",
+					       dirname, temp, xsvals.mask_ext);
+	      g_free (dirname);
+	    }
+	  else
+	    {
+	      mask_filename = g_strdup_printf ("%s%s.xbm",
+					       temp, xsvals.mask_ext);
+	    }
+
+	  g_free (temp);
+
+	  /* Change any non-alphanumeric prefix characters to underscores. */
+	  temp = xsvals.prefix;
+	  while (*temp)
+	    {
+	      if (!isalnum (*temp))
+		*temp = '_';
+	      temp ++;
+	    }
+
+	  mask_prefix = g_strdup_printf ("%s%s", xsvals.prefix, xsvals.mask_ext);
+
+	  if (save_image (param[3].data.d_string,
+			  xsvals.prefix,
+			  xsvals.comment,
+			  FALSE,
+			  image_ID, drawable_ID) &&
+
+	      xsvals.write_mask &&
+
+	      save_image (mask_filename,
+			  mask_prefix,
+			  xsvals.comment,
+			  TRUE,
+			  image_ID, drawable_ID))
 	    {
 	      /*  Store xsvals data  */
 	      gimp_set_data ("file_xbm_save", &xsvals, sizeof (xsvals));
@@ -364,6 +464,9 @@ run (gchar   *name,
 	    {
 	      status = STATUS_EXECUTION_ERROR;
 	    }
+
+	  g_free (mask_prefix);
+	  g_free (mask_filename);
 	}
 
       if (export == EXPORT_EXPORT)
@@ -398,6 +501,80 @@ getval (gint c,
 }
 
 
+/* Get a comment */
+static gchar *
+fgetcomment (FILE *fp)
+{
+  GString *str = NULL;
+  gint comment, c;
+
+  comment = 0;
+  do
+    {
+      c = fgetc (fp);
+      if (comment)
+	{
+	  if (c == '*')
+	    {
+	      /* In a comment, with potential to leave. */
+	      comment = 1;
+	    }
+	  else if (comment == 1 && c == '/')
+	    {
+	      gchar *retval;
+
+	      /* Leaving a comment. */
+	      comment = 0;
+
+	      retval = g_strstrip (g_strdup (str->str));
+	      g_string_free (str, TRUE);
+	      return retval;
+	    }
+	  else
+	    {
+	      /* In a comment, with no potential to leave. */
+	      comment = 2;
+	      g_string_append_c (str, c);
+	    }
+	}
+      else
+	{
+	  /* Not in a comment. */
+	  if (c == '/')
+	    {
+	      /* Potential to enter a comment. */
+	      c = fgetc (fp);
+	      if (c == '*')
+		{
+		  /* Entered a comment, with no potential to leave. */
+		  comment = 2;
+		  str = g_string_new (NULL);
+		}
+	      else
+		{
+		  /* put everything back and return */
+		  ungetc (c, fp);
+		  c = '/';
+		  ungetc (c, fp);
+		  return NULL;
+		}
+	    }
+	  else if (isspace (c))
+	    {
+	      /* Skip leading whitespace */
+	      continue;
+	    }
+	}
+    }
+  while (comment && c != EOF);
+
+  if (str)
+    g_string_free (str, TRUE);
+
+  return NULL;
+}
+
+
 /* Same as fgetc, but skip C-style comments and insert whitespace. */
 static gint
 cpp_fgetc (FILE *fp)
@@ -528,14 +705,19 @@ load_image (gchar *filename)
   FILE *fp;
   gint32 image_ID, layer_ID;
 
-  GPixelRgn pixel_rgn;
+  GPixelRgn  pixel_rgn;
   GDrawable *drawable;
   guchar *data;
-  int width, height, intbits;
-  int c, i, j, k;
-  int tileheight, rowoffset;
+  gint    intbits;
+  gint    width = 0;
+  gint    height = 0;
+  gint    x_hot = 0;
+  gint    y_hot = 0;
+  gint    c, i, j, k;
+  gint    tileheight, rowoffset;
 
-  char *name_buf;
+  gchar *name_buf;
+  gchar *comment;
 
   guchar cmap[] =
   {
@@ -554,12 +736,13 @@ load_image (gchar *filename)
   gimp_progress_init (name_buf);
   g_free (name_buf);
 
+  comment = fgetcomment (fp);
+
   /* Loosely parse the header */
   intbits = height = width = 0;
   c = ' ';
   do
     {
-
       if (isspace (c))
 	{
 	  if (match (fp, "char"))
@@ -602,6 +785,24 @@ load_image (gchar *filename)
 		  continue;
 		}
 	    }
+	  else if (match (fp, "x_hot"))
+	    {
+	      c = fgetc (fp);
+	      if (isspace (c))
+		{
+		  x_hot = get_int (fp);
+		  continue;
+		}
+	    }
+	  else if (match (fp, "y_hot"))
+	    {
+	      c = fgetc (fp);
+	      if (isspace (c))
+		{
+		  y_hot = get_int (fp);
+		  continue;
+		}
+	    }
 	}
 
       c = cpp_fgetc (fp);
@@ -635,6 +836,36 @@ load_image (gchar *filename)
   image_ID = gimp_image_new (width, height, INDEXED);
   gimp_image_set_filename (image_ID, filename);
 
+  if (comment)
+    {
+      GimpParasite *parasite;
+
+      parasite = gimp_parasite_new ("gimp-comment",
+				    GIMP_PARASITE_PERSISTENT,
+				    strlen (comment) + 1, (gpointer) comment);
+      gimp_image_parasite_attach (image_ID, parasite);
+      gimp_parasite_free (parasite);
+
+      g_free (comment);
+    }
+
+  x_hot = CLAMP (x_hot, 0, width);
+  y_hot = CLAMP (y_hot, 0, height);
+
+  if (x_hot > 0 || y_hot > 0)
+    {
+      GimpParasite *parasite;
+      gchar        *str;
+
+      str = g_strdup_printf ("%d %d", x_hot, y_hot);
+      parasite = gimp_parasite_new ("hot-spot",
+				    GIMP_PARASITE_PERSISTENT,
+				    strlen (str) + 1, (gpointer) str);
+      g_free (str);
+      gimp_image_parasite_attach (image_ID, parasite);
+      gimp_parasite_free (parasite);
+    }
+
   /* Set a black-and-white colormap. */
   gimp_image_set_cmap (image_ID, cmap, 2);
 
@@ -702,91 +933,53 @@ load_image (gchar *filename)
   return image_ID;
 }
 
-
-static int gtk_initialized = FALSE;
-
-
-static void
-not_bw_dialog (void)
+static gboolean
+save_image (gchar    *filename,
+	    gchar    *prefix,
+	    gchar    *comment,
+	    gboolean  save_mask,
+	    gint32    image_ID,
+	    gint32    drawable_ID)
 {
-  GtkWidget *dlg, *label, *frame, *vbox;
-
-  if (!gtk_initialized)
-    {
-      fprintf (stderr, "XBM: can only save two color indexed images\n");
-      return;
-    }
-
-  dlg = gimp_dialog_new (_("XBM Warning"), "xbm",
-			 gimp_plugin_help_func, "filters/xbm.html",
-			 GTK_WIN_POS_MOUSE,
-			 FALSE, TRUE, FALSE,
-
-			 _("Cancel"), gtk_main_quit,
-			 NULL, NULL, NULL, TRUE, TRUE,
-
-			 NULL);
-
-  /*  the warning message  */
-  frame = gtk_frame_new (NULL);
-  gtk_frame_set_shadow_type (GTK_FRAME (frame), GTK_SHADOW_ETCHED_IN);
-  gtk_container_border_width (GTK_CONTAINER (frame), 6);
-  gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dlg)->vbox), frame,
-		      TRUE, TRUE, 0);
-
-  vbox = gtk_vbox_new (FALSE, 5);
-  gtk_container_border_width (GTK_CONTAINER (vbox), 4);
-  gtk_container_add (GTK_CONTAINER (frame), vbox);
-
-  label = gtk_label_new (_("The image which you are trying to save as\n"
-			   "an XBM contains more than two colors.\n\n"
-			   "Please convert it to a black and white\n"
-			   "(1-bit) indexed image and try again."));
-  gtk_box_pack_start (GTK_BOX (vbox), label, TRUE, TRUE, 0);
-
-  gtk_widget_show (label);
-  gtk_widget_show (vbox);
-  gtk_widget_show (frame);
-  gtk_widget_show (dlg);
-
-  gtk_main ();
-  gdk_flush ();
-}
-
-
-static gint
-save_image (gchar  *filename,
-	    gint32  image_ID,
-	    gint32  drawable_ID)
-{
-  GDrawable *drawable;
-  GPixelRgn pixel_rgn;
+  GimpDrawable *drawable;
+  GimpPixelRgn  pixel_rgn;
   FILE *fp;
 
-  int width, height, colors, dark;
-  int intbits, lineints, need_comma, nints, rowoffset, tileheight;
-  int c, i, j, k, thisbit;
+  gint width, height, colors, dark;
+  gint intbits, lineints, need_comma, nints, rowoffset, tileheight;
+  gint c, i, j, k, thisbit;
 
-  guchar *data, *cmap, *name_buf;
-  char *prefix, *p, *intfmt;
+  gboolean has_alpha;
+  gint     bpp;
+
+  guchar *data, *cmap, *name_buf, *intfmt;
 
   drawable = gimp_drawable_get (drawable_ID);
-  width = drawable->width;
+  width  = drawable->width;
   height = drawable->height;
   cmap = gimp_image_get_cmap (image_ID, &colors);
 
-#if 0
-  printf ("XBM: run `gdb xbm' and `attach %d'\n", getpid ());
-  kill (getpid (), 19);
-#endif
-
-  if (gimp_drawable_type (drawable_ID) != INDEXED_IMAGE || colors > 2)
+  if (!gimp_drawable_is_indexed (drawable_ID) || colors > 2)
     {
       /* The image is not black-and-white. */
-      not_bw_dialog ();
+      g_message (_("The image which you are trying to save as\n"
+		   "an XBM contains more than two colors.\n\n"
+		   "Please convert it to a black and white\n"
+		   "(1-bit) indexed image and try again."));
       return FALSE;
     }
 
+  has_alpha = gimp_drawable_has_alpha (drawable_ID);
+
+  if (!has_alpha && save_mask)
+    {
+      g_message (_("You cannot save a cursor mask for an image\n"
+		   "which has no alpha channel."));
+      return FALSE;
+    }
+
+  bpp = gimp_drawable_bpp (drawable_ID);
+
   name_buf = g_strdup_printf (_("Saving %s:"), filename);
   gimp_progress_init (name_buf);
   g_free (name_buf);
@@ -795,10 +988,10 @@ save_image (gchar  *filename,
   dark = 0;
   if (colors > 1)
     {
-      int first, second;
+      gint first, second;
 
       /* Maybe the second color is darker than the first. */
-      first = (cmap[0] * cmap[0]) + (cmap[1] * cmap[1]) + (cmap[2] * cmap[2]);
+      first  = (cmap[0] * cmap[0]) + (cmap[1] * cmap[1]) + (cmap[2] * cmap[2]);
       second = (cmap[3] * cmap[3]) + (cmap[4] * cmap[4]) + (cmap[5] * cmap[5]);
 
       if (second < first)
@@ -814,25 +1007,15 @@ save_image (gchar  *filename,
     }
 
   /* Maybe write the image comment. */
-  if (*xsvals.comment)
-    fprintf (fp, "/* %s */\n", xsvals.comment);
-
-  /* Change any non-alphanumeric prefix characters to underscores. */
-  prefix = xsvals.prefix;
-  p = prefix;
-  while (*p)
-    {
-      if (!isalnum (*p))
-	*p = '_';
-      p ++;
-    }
+  if (*comment)
+    fprintf (fp, "/* %s */\n", comment);
 
   /* Write out the image height and width. */
-  fprintf (fp, "#define %s_width %d\n", prefix, width);
+  fprintf (fp, "#define %s_width %d\n",  prefix, width);
   fprintf (fp, "#define %s_height %d\n", prefix, height);
 
   /* Write out the hotspot, if any. */
-  if (xsvals.x_hot >= 0 && xsvals.y_hot >= 0)
+  if (xsvals.use_hot)
     {
       fprintf (fp, "#define %s_x_hot %d\n", prefix, xsvals.x_hot);
       fprintf (fp, "#define %s_y_hot %d\n", prefix, xsvals.y_hot);
@@ -855,11 +1038,11 @@ save_image (gchar  *filename,
     }
 
   fprintf (fp, "static %s %s_bits[] = {\n  ",
-	   xsvals.x10_format ? "short" : "char", prefix);
+	   xsvals.x10_format ? "unsigned short" : "unsigned char", prefix);
 
   /* Allocate a new set of pixels. */
   tileheight = gimp_tile_height ();
-  data = (guchar *) g_malloc(width * tileheight);
+  data = (guchar *) g_malloc (width * tileheight * bpp);
 
   gimp_pixel_rgn_init (&pixel_rgn, drawable, 0, 0, width, height,
 		       FALSE, FALSE);
@@ -882,11 +1065,11 @@ save_image (gchar  *filename,
       for (j = 0; j < tileheight; j ++)
 	{
 	  /* Write out a row at a time. */
-	  rowoffset = j * width;
+	  rowoffset = j * width * bpp;
 	  c = 0;
 	  thisbit = 0;
 
-	  for (k = 0; k < width; k ++)
+	  for (k = 0; k < width * bpp; k += bpp)
 	    {
 	      if (k != 0 && thisbit == intbits)
 		{
@@ -909,7 +1092,17 @@ save_image (gchar  *filename,
 		}
 
 	      /* Pack INTBITS pixels into an integer. */
-	      c |= ((data[rowoffset + k] == dark) ? 1 : 0) << (thisbit ++);
+	      if (save_mask)
+		{
+		  c |= ((data[rowoffset + k + 1] < 128) ? 0 : 1) << (thisbit ++);
+		}
+	      else
+		{
+		  if (has_alpha && (data[rowoffset + k + 1] < 128))
+		    c |= 0 << (thisbit ++);
+		  else
+		    c |= ((data[rowoffset + k] == dark) ? 1 : 0) << (thisbit ++);
+		}
 	    }
 
 	  if (thisbit != 0)
@@ -938,23 +1131,6 @@ save_image (gchar  *filename,
   return TRUE;
 }
 
-
-static void 
-init_gtk (void)
-{
-  gchar **argv;
-  gint    argc;
-
-  argc    = 1;
-  argv    = g_new (gchar *, 1);
-  argv[0] = g_strdup ("xbm");
-  
-  gtk_init (&argc, &argv);
-  gtk_rc_parse (gimp_gtkrc ());
-
-  gtk_initialized = TRUE;
-}
-
 static gint
 save_dialog (gint32 drawable_ID)
 {
@@ -964,9 +1140,11 @@ save_dialog (gint32 drawable_ID)
   GtkWidget *toggle;
   GtkWidget *table;
   GtkWidget *entry;
+  GtkWidget *spinbutton;
+  GtkObject *adj;
 
   dlg = gimp_dialog_new (_("Save as XBM"), "xbm",
-			 gimp_plugin_help_func, "filters/xbm.html",
+			 gimp_standard_help_func, "filters/xbm.html",
 			 GTK_WIN_POS_MOUSE,
 			 FALSE, TRUE, FALSE,
 
@@ -1028,6 +1206,78 @@ save_dialog (gint32 drawable_ID)
                       GTK_SIGNAL_FUNC (comment_entry_callback),
                       NULL);
 
+  /* hotspot toggle */
+  toggle = gtk_check_button_new_with_label (_("Write Hot Spot Values"));
+  gtk_box_pack_start (GTK_BOX (vbox), toggle, FALSE, FALSE, 0);
+  gtk_signal_connect (GTK_OBJECT (toggle), "toggled",
+		      GTK_SIGNAL_FUNC (gimp_toggle_button_update),
+		      &xsvals.use_hot);
+  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (toggle), xsvals.use_hot);
+  gtk_widget_show (toggle);
+
+  table = gtk_table_new (2, 2, FALSE);
+  gtk_table_set_col_spacings (GTK_TABLE (table), 4);
+  gtk_table_set_row_spacings (GTK_TABLE (table), 2);
+  gtk_box_pack_start (GTK_BOX (vbox), table, FALSE, FALSE, 0);
+  gtk_widget_show (table);
+
+  gtk_object_set_data (GTK_OBJECT (toggle), "set_sensitive", table);
+  gtk_widget_set_sensitive (table, xsvals.use_hot);
+
+  spinbutton = gimp_spin_button_new (&adj, xsvals.x_hot, 0,
+				     gimp_drawable_width (drawable_ID) - 1,
+				     1, 1, 1, 0, 0);
+  gtk_signal_connect (GTK_OBJECT (adj), "value_changed",
+		      GTK_SIGNAL_FUNC (gimp_int_adjustment_update),
+		      &xsvals.x_hot);
+  gimp_table_attach_aligned (GTK_TABLE (table), 0, 0,
+			     _("Hot Spot X:"), 1.0, 0.5,
+			     spinbutton, 1, TRUE);
+
+  spinbutton = gimp_spin_button_new (&adj, xsvals.y_hot, 0,
+				     gimp_drawable_height (drawable_ID) - 1,
+				     1, 1, 1, 0, 0);
+  gtk_signal_connect (GTK_OBJECT (adj), "value_changed",
+		      GTK_SIGNAL_FUNC (gimp_int_adjustment_update),
+		      &xsvals.y_hot);
+  gimp_table_attach_aligned (GTK_TABLE (table), 0, 1,
+			     _("Y:"), 1.0, 0.5,
+			     spinbutton, 1, TRUE);
+
+  /* mask file */
+  frame = gtk_frame_new (_("Mask File"));
+  gtk_box_pack_start (GTK_BOX (vbox), frame, FALSE, FALSE, 0);
+  gtk_widget_show (frame);
+
+  table = gtk_table_new (2, 2, FALSE);
+  gtk_container_set_border_width (GTK_CONTAINER (table), 2);
+  gtk_table_set_col_spacings (GTK_TABLE (table), 4);
+  gtk_table_set_row_spacings (GTK_TABLE (table), 2);
+  gtk_container_add (GTK_CONTAINER (frame), table);
+  gtk_widget_show (table);
+
+  toggle = gtk_check_button_new_with_label (_("Write Extra Mask File"));
+  gtk_table_attach_defaults (GTK_TABLE (table), toggle, 0, 2, 0, 1);
+  gtk_signal_connect (GTK_OBJECT (toggle), "toggled",
+		      GTK_SIGNAL_FUNC (gimp_toggle_button_update),
+		      &xsvals.write_mask);
+  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (toggle), xsvals.write_mask);
+  gtk_widget_show (toggle);
+
+  entry = gtk_entry_new_with_max_length (MAX_MASK_EXT);
+  gtk_entry_set_text (GTK_ENTRY (entry), xsvals.mask_ext);
+  gimp_table_attach_aligned (GTK_TABLE (table), 0, 1,
+			     _("Mask File Extension:"), 1.0, 0.5,
+			     entry, 1, TRUE);
+  gtk_signal_connect (GTK_OBJECT (entry), "changed",
+                      GTK_SIGNAL_FUNC (mask_ext_entry_callback),
+                      NULL);
+
+  gtk_object_set_data (GTK_OBJECT (toggle), "set_sensitive", entry);
+  gtk_widget_set_sensitive (entry, xsvals.write_mask);
+
+  gtk_widget_set_sensitive (frame, gimp_drawable_has_alpha (drawable_ID));
+
   /* Done. */
   gtk_widget_show (vbox);
   gtk_widget_show (dlg);
@@ -1058,6 +1308,15 @@ prefix_entry_callback (GtkWidget *widget,
 	   gtk_entry_get_text (GTK_ENTRY (widget)), MAX_PREFIX);
 }
 
+static void
+mask_ext_entry_callback (GtkWidget *widget,
+		       gpointer   data)
+{
+  memset (xsvals.prefix, 0, sizeof (xsvals.mask_ext));
+  strncpy (xsvals.mask_ext,
+	   gtk_entry_get_text (GTK_ENTRY (widget)), MAX_MASK_EXT);
+}
+
 static void
 save_ok_callback (GtkWidget *widget,
 		  gpointer   data)
