@@ -1,5 +1,5 @@
 /*
- * "$Id: print-ps.c,v 1.19 2000/01/28 13:12:50 neo Exp $"
+ * "$Id: print-ps.c,v 1.21 2000/08/15 00:43:47 mitch Exp $"
  *
  *   Print plug-in Adobe PostScript driver for the GIMP.
  *
@@ -37,10 +37,12 @@
 
 #include "print.h"
 #include <time.h>
-#include <glib.h>
+#include <string.h>
+#include <limits.h>
 
 /*#define DEBUG*/
 
+
 /*
  * Local variables...
  */
@@ -55,34 +57,24 @@ static char	*ps_ppd_file = NULL;
 
 static void	ps_hex(FILE *, unsigned short *, int);
 static void	ps_ascii85(FILE *, unsigned short *, int, int);
-static char	*ppd_find(char *, char *, char *, int *);
+static char	*ppd_find(const char *, const char *, const char *, int *);
 
 
 /*
  * 'ps_parameters()' - Return the parameter values for the given parameter.
  */
 
-char **				/* O - Parameter values */
-ps_parameters(int  model,	/* I - Printer model */
-              char *ppd_file,	/* I - PPD file (not used) */
-              char *name,	/* I - Name of parameter */
-              int  *count)	/* O - Number of values */
+char **					/* O - Parameter values */
+ps_parameters(const printer_t *printer,	/* I - Printer model */
+              char *ppd_file,		/* I - PPD file (not used) */
+              char *name,		/* I - Name of parameter */
+              int  *count)		/* O - Number of values */
 {
   int		i;
   char		line[1024],
 		lname[255],
 		loption[255];
   char		**valptrs;
-  static char	*media_sizes[] =
-		{
-		  ("Letter"),
-		  ("Legal"),
-		  ("A4"),
-		  ("Tabloid"),
-		  ("A3"),
-		  ("12x18")
-		};
-
 
   if (count == NULL)
     return (NULL);
@@ -106,24 +98,26 @@ ps_parameters(int  model,	/* I - Printer model */
   }
 
   if (ps_ppd == NULL)
-  {
-    if (strcmp(name, "PageSize") == 0)
     {
-      *count = 6;
-
-      valptrs = malloc(*count * sizeof(char *));
-      for (i = 0; i < *count; i ++)
+      if (strcmp(name, "PageSize") == 0)
 	{
-	  /* strdup doesn't appear to be POSIX... */
-	  valptrs[i] = malloc(strlen(media_sizes[i]) + 1);
-	  strcpy(valptrs[i], media_sizes[i]);
+	  const papersize_t *papersizes = get_papersizes();
+	  valptrs = malloc(sizeof(char *) * known_papersizes());
+	  *count = 0;
+	  for (i = 0; i < known_papersizes(); i++)
+	    {
+	      if (strlen(papersizes[i].name) > 0)
+		{
+		  valptrs[*count] = malloc(strlen(papersizes[i].name) + 1);
+		  strcpy(valptrs[*count], papersizes[i].name);
+		  (*count)++;
+		}
+	    }
+	  return (valptrs);
 	}
-
-      return (valptrs);
+      else
+	return (NULL);
     }
-    else
-      return (NULL);
-  }
 
   rewind(ps_ppd);
   *count = 0;
@@ -138,7 +132,7 @@ ps_parameters(int  model,	/* I - Printer model */
     if (sscanf(line, "*%s %[^/:]", lname, loption) != 2)
       continue;
 
-    if (g_strcasecmp(lname, name) == 0)
+    if (strcasecmp(lname, name) == 0)
     {
       valptrs[(*count)] = malloc(strlen(loption) + 1);
       strcpy(valptrs[(*count)], loption);
@@ -161,9 +155,8 @@ ps_parameters(int  model,	/* I - Printer model */
  */
 
 void
-ps_media_size(int  model,		/* I - Printer model */
-              char *ppd_file,		/* I - PPD file (not used) */
-              char *media_size,		/* I - Media size */
+ps_media_size(const printer_t *printer,	/* I - Printer model */
+	      const vars_t *v,		/* I */
               int  *width,		/* O - Width in points */
               int  *length)		/* O - Length in points */
 {
@@ -175,10 +168,12 @@ ps_media_size(int  model,		/* I - Printer model */
          media_size, width, length);
 #endif /* DEBUG */
 
-  if ((dimensions = ppd_find(ppd_file, "PaperDimension", media_size, NULL)) != NULL)
+  if ((dimensions = ppd_find(v->ppd_file, "PaperDimension", v->media_size,
+			     NULL))
+      != NULL)
     sscanf(dimensions, "%d%d", width, length);
   else
-    default_media_size(model, ppd_file, media_size, width, length);
+    default_media_size(printer, v, width, length);
 }
 
 
@@ -187,9 +182,8 @@ ps_media_size(int  model,		/* I - Printer model */
  */
 
 void
-ps_imageable_area(int  model,		/* I - Printer model */
-                  char *ppd_file,	/* I - PPD file (not used) */
-                  char *media_size,	/* I - Media size */
+ps_imageable_area(const printer_t *printer,	/* I - Printer model */
+		  const vars_t *v,      /* I */
                   int  *left,		/* O - Left position in points */
                   int  *right,		/* O - Right position in points */
                   int  *bottom,		/* O - Bottom position in points */
@@ -202,7 +196,8 @@ ps_imageable_area(int  model,		/* I - Printer model */
 	ftop;
 
 
-  if ((area = ppd_find(ppd_file, "ImageableArea", media_size, NULL)) != NULL)
+  if ((area = ppd_find(v->ppd_file, "ImageableArea", v->media_size, NULL))
+      != NULL)
   {
 #ifdef DEBUG
     printf("area = \'%s\'\n", area);
@@ -219,7 +214,7 @@ ps_imageable_area(int  model,		/* I - Printer model */
   }
   else
   {
-    default_media_size(model, ppd_file, media_size, right, top);
+    default_media_size(printer, v, right, top);
     *left   = 18;
     *right  -= 18;
     *top    -= 36;
@@ -227,20 +222,35 @@ ps_imageable_area(int  model,		/* I - Printer model */
   }
 }
 
+void
+ps_limit(const printer_t *printer,	/* I - Printer model */
+	    const vars_t *v,  		/* I */
+	    int  *width,		/* O - Left position in points */
+	    int  *length)		/* O - Top position in points */
+{
+  *width =	INT_MAX;
+    *length =	INT_MAX;
+}
+
+const char *
+ps_default_resolution(const printer_t *printer)
+{
+  return "default";
+}
 
 /*
  * 'ps_print()' - Print an image to a PostScript printer.
  */
 
 void
-ps_print(int       model,		/* I - Model (Level 1 or 2) */
+ps_print(const printer_t *printer,		/* I - Model (Level 1 or 2) */
          int       copies,		/* I - Number of copies */
          FILE      *prn,		/* I - File to print to */
          Image     image,		/* I - Image to print */
-	 unsigned char    *cmap,	/* I - Colormap (for indexed images) */
-	 lut_t     *lut,		/* I - Brightness lookup table */
-	 vars_t    *v)
+	 const vars_t    *v)
 {
+  unsigned char *cmap = v->cmap;
+  int		model = printer->model;
   char 		*ppd_file = v->ppd_file;
   char 		*resolution = v->resolution;
   char 		*media_size = v->media_size;
@@ -252,7 +262,7 @@ ps_print(int       model,		/* I - Model (Level 1 or 2) */
   int		top = v->top;
   int		left = v->left;
   int		i, j;		/* Looping vars */
-  int		x, y;		/* Looping vars */
+  int		y;		/* Looping vars */
   unsigned char	*in;		/* Input pixels from image */
   unsigned short	*out;		/* Output pixels for printer */
   int		page_left,	/* Left margin of page */
@@ -265,10 +275,7 @@ ps_print(int       model,		/* I - Model (Level 1 or 2) */
 		out_height,	/* Height of image on page */
 		out_bpp,	/* Output bytes per pixel */
 		out_length,	/* Output length (Level 2 output) */
-		out_offset,	/* Output offset (Level 2 output) */
-		temp_width,	/* Temporary width of image on page */
-		temp_height,	/* Temporary height of image on page */
-		landscape;	/* True if we rotate the output 90 degrees */
+		out_offset;	/* Output offset (Level 2 output) */
   time_t	curtime;	/* Current time of day */
   convert_t	colorfunc;	/* Color conversion function... */
   char		*command;	/* PostScript command */
@@ -282,8 +289,9 @@ ps_print(int       model,		/* I - Model (Level 1 or 2) */
   int           image_height,
                 image_width,
                 image_bpp;
+  vars_t	nv;
 
-
+  memcpy(&nv, v, sizeof(vars_t));
  /*
   * Setup a read-only pixel region for the entire image...
   */
@@ -297,141 +305,28 @@ ps_print(int       model,		/* I - Model (Level 1 or 2) */
   * Choose the correct color conversion function...
   */
 
-  if (image_bpp < 3 && cmap == NULL)
-    output_type = OUTPUT_GRAY;		/* Force grayscale output */
+  if (image_bpp < 3 && cmap == NULL && output_type == OUTPUT_COLOR)
+    output_type = OUTPUT_GRAY_COLOR;		/* Force grayscale output */
 
-  if (output_type == OUTPUT_COLOR)
-  {
-    out_bpp = 3;
-
-    if (image_bpp >= 3)
-      colorfunc = rgb_to_rgb;
-    else
-      colorfunc = indexed_to_rgb;
-  }
-  else
-  {
-    out_bpp = 1;
-
-    if (image_bpp >= 3)
-      colorfunc = rgb_to_gray;
-    else if (cmap == NULL)
-      colorfunc = gray_to_gray;
-    else
-      colorfunc = indexed_to_gray;
-  }
+  colorfunc = choose_colorfunc(output_type, image_bpp, cmap, &out_bpp, &nv);
 
  /*
   * Compute the output size...
   */
 
-  landscape = 0;
-  ps_imageable_area(model, ppd_file, media_size, &page_left, &page_right,
+  ps_imageable_area(printer, &nv, &page_left, &page_right,
                     &page_bottom, &page_top);
+  compute_page_parameters(page_right, page_left, page_top, page_bottom,
+			  scaling, image_width, image_height, image,
+			  &orientation, &page_width, &page_height,
+			  &out_width, &out_height, &left, &top);
 
-  page_width  = page_right - page_left;
-  page_height = page_top - page_bottom;
-
-#ifdef DEBUG
-  printf("page_width = %d, page_height = %d\n", page_width, page_height);
-  printf("image_width = %d, image_height = %d\n", image_width, image_height);
-  printf("scaling = %.1f\n", scaling);
-#endif /* DEBUG */
-
- /*
-  * Portrait width/height...
-  */
-
-  if (scaling < 0.0)
-  {
-   /*
-    * Scale to pixels per inch...
-    */
-
-    out_width  = image_width * -72.0 / scaling;
-    out_height = image_height * -72.0 / scaling;
-  }
-  else
-  {
-   /*
-    * Scale by percent...
-    */
-
-    out_width  = page_width * scaling / 100.0;
-    out_height = out_width * image_height / image_width;
-    if (out_height > page_height)
-    {
-      out_height = page_height * scaling / 100.0;
-      out_width  = out_height * image_width / image_height;
-    }
-  }
-
- /*
-  * Landscape width/height...
-  */
-
-  if (scaling < 0.0)
-  {
-   /*
-    * Scale to pixels per inch...
-    */
-
-    temp_width  = image_height * -72.0 / scaling;
-    temp_height = image_width * -72.0 / scaling;
-  }
-  else
-  {
-   /*
-    * Scale by percent...
-    */
-
-    temp_width  = page_width * scaling / 100.0;
-    temp_height = temp_width * image_width / image_height;
-    if (temp_height > page_height)
-    {
-      temp_height = page_height;
-      temp_width  = temp_height * image_height / image_width;
-    }
-  }
-
- /*
-  * See which orientation has the greatest area (or if we need to rotate the
-  * image to fit it on the page...)
-  */
-
-  if (orientation == ORIENT_AUTO)
-  {
-    if (scaling < 0.0)
-    {
-      if ((out_width > page_width && out_height < page_width) ||
-          (out_height > page_height && out_width < page_height))
-	orientation = ORIENT_LANDSCAPE;
-      else
-	orientation = ORIENT_PORTRAIT;
-    }
-    else
-    {
-      if ((temp_width * temp_height) > (out_width * out_height))
-	orientation = ORIENT_LANDSCAPE;
-      else
-	orientation = ORIENT_PORTRAIT;
-    }
-  }
-
-  if (orientation == ORIENT_LANDSCAPE)
-  {
-    out_width  = temp_width;
-    out_height = temp_height;
-    landscape  = 1;
-
-   /*
-    * Swap left/top offsets...
-    */
-
-    x    = top;
-    top  = left;
-    left = x;
-  }
+  /*
+   * Recompute the image height and width.  If the image has been
+   * rotated, these will change from previously.
+   */
+  image_height = Image_height(image);
+  image_width = Image_width(image);
 
  /*
   * Let the user know what we're doing...
@@ -447,6 +342,8 @@ ps_print(int       model,		/* I - Model (Level 1 or 2) */
 
   if (left < 0)
     left = (page_width - out_width) / 2 + page_left;
+  else
+    left += page_left;
 
   if (top < 0)
     top  = (page_height + out_height) / 2 + page_bottom;
@@ -464,9 +361,9 @@ ps_print(int       model,		/* I - Model (Level 1 or 2) */
   _fsetmode(prn, "t");
 #endif
   fputs("%!PS-Adobe-3.0\n", prn);
-  fprintf(prn, "%%Creator: %s\n", Image_get_pluginname(image));
+  fprintf(prn, "%%%%Creator: %s\n", Image_get_appname(image));
   fprintf(prn, "%%%%CreationDate: %s", ctime(&curtime));
-  fputs("%%Copyright: 1997-1999 by Michael Sweet (mike@easysw.com) and Robert Krawitz (rlk@alum.mit.edu)\n", prn);
+  fputs("%%Copyright: 1997-2000 by Michael Sweet (mike@easysw.com) and Robert Krawitz (rlk@alum.mit.edu)\n", prn);
   fprintf(prn, "%%%%BoundingBox: %d %d %d %d\n",
           left, top - out_height, left + out_width, top);
   fputs("%%DocumentData: Clean7Bit\n", prn);
@@ -540,6 +437,7 @@ ps_print(int       model,		/* I - Model (Level 1 or 2) */
     for (i = 0; i < num_commands; i ++)
     {
       fputs(commands[i].command, prn);
+      fputs("\n", prn);
       free(commands[i].command);
     }
 
@@ -547,40 +445,29 @@ ps_print(int       model,		/* I - Model (Level 1 or 2) */
   }
 
  /*
-  * Output the page, rotating as necessary...
+  * Output the page...
   */
 
   fputs("%%Page: 1\n", prn);
   fputs("gsave\n", prn);
 
-  if (landscape)
-  {
-    fprintf(prn, "%d %d translate\n", left, top - out_height);
-    fprintf(prn, "%.3f %.3f scale\n",
-            (float)out_width / ((float)image_height),
-            (float)out_height / ((float)image_width));
-  }
-  else
-  {
-    fprintf(prn, "%d %d translate\n", left, top);
-    fprintf(prn, "%.3f %.3f scale\n",
-            (float)out_width / ((float)image_width),
-            (float)out_height / ((float)image_height));
-  }
+  fprintf(prn, "%d %d translate\n", left, top);
+  fprintf(prn, "%.3f %.3f scale\n",
+          (float)out_width / ((float)image_width),
+          (float)out_height / ((float)image_height));
 
   in  = malloc(image_width * image_bpp);
   out = malloc((image_width * out_bpp + 3) * 2);
 
+  compute_lut(256, &nv);
+
   if (model == 0)
   {
     fprintf(prn, "/picture %d string def\n", image_width * out_bpp);
 
     fprintf(prn, "%d %d 8\n", image_width, image_height);
 
-    if (landscape)
-      fputs("[ 0 1 1 0 0 0 ]\n", prn);
-    else
-      fputs("[ 1 0 0 -1 0 1 ]\n", prn);
+    fputs("[ 1 0 0 -1 0 1 ]\n", prn);
 
     if (output_type == OUTPUT_GRAY)
       fputs("{currentfile picture readhexstring pop} image\n", prn);
@@ -593,7 +480,7 @@ ps_print(int       model,		/* I - Model (Level 1 or 2) */
 	Image_note_progress(image, y, image_height);
 
       Image_get_row(image, in, y);
-      (*colorfunc)(in, out, image_width, image_bpp, lut, cmap, v);
+      (*colorfunc)(in, out, image_width, image_bpp, cmap, &nv);
 
       ps_hex(prn, out, image_width * out_bpp);
     }
@@ -622,10 +509,7 @@ ps_print(int       model,		/* I - Model (Level 1 or 2) */
     if ((image_width * 72 / out_width) < 100)
       fputs("\t/Interpolate true\n", prn);
 
-    if (landscape)
-      fputs("\t/ImageMatrix [ 0 1 1 0 0 0 ]\n", prn);
-    else
-      fputs("\t/ImageMatrix [ 1 0 0 -1 0 1 ]\n", prn);
+    fputs("\t/ImageMatrix [ 1 0 0 -1 0 1 ]\n", prn);
 
     fputs(">>\n", prn);
     fputs("image\n", prn);
@@ -636,7 +520,7 @@ ps_print(int       model,		/* I - Model (Level 1 or 2) */
 	Image_note_progress(image, y, image_height);
 
       Image_get_row(image, in, y);
-      (*colorfunc)(in, out + out_offset, image_width, image_bpp, lut, cmap, v);
+      (*colorfunc)(in, out + out_offset, image_width, image_bpp, cmap, &nv);
 
       out_length = out_offset + image_width * out_bpp;
 
@@ -655,7 +539,9 @@ ps_print(int       model,		/* I - Model (Level 1 or 2) */
         memcpy(out, out + out_length - out_offset, out_offset);
     }
   }
+  Image_progress_conclude(image);
 
+  free_lut(&nv);
   free(in);
   free(out);
 
@@ -777,20 +663,22 @@ ps_ascii85(FILE   *prn,		/* I - File to print to */
  */
 
 static char *			/* O - Control string */
-ppd_find(char *ppd_file,	/* I - Name of PPD file */
-         char *name,		/* I - Name of parameter */
-         char *option,		/* I - Value of parameter */
+ppd_find(const char *ppd_file,	/* I - Name of PPD file */
+         const char *name,		/* I - Name of parameter */
+         const char *option,		/* I - Value of parameter */
          int  *order)		/* O - Order of the control string */
 {
   char		line[1024],	/* Line from file */
 		lname[255],	/* Name from line */
 		loption[255],	/* Value from line */
 		*opt;		/* Current control string pointer */
-  static char	value[32768];	/* Current control string value */
+  static char	*value = NULL;	/* Current control string value */
 
 
   if (ppd_file == NULL || name == NULL || option == NULL)
     return (NULL);
+  if (!value)
+    value = malloc(32768);
 
   if (ps_ppd_file == NULL || strcmp(ps_ppd_file, ppd_file) != 0)
   {
@@ -817,7 +705,7 @@ ppd_find(char *ppd_file,	/* I - Name of PPD file */
     if (line[0] != '*')
       continue;
 
-    if (g_strncasecmp(line, "*OrderDependency:", 17) == 0 && order != NULL)
+    if (strncasecmp(line, "*OrderDependency:", 17) == 0 && order != NULL)
     {
       sscanf(line, "%*s%d", order);
       continue;
@@ -825,8 +713,8 @@ ppd_find(char *ppd_file,	/* I - Name of PPD file */
     else if (sscanf(line, "*%s %[^/:]", lname, loption) != 2)
       continue;
 
-    if (g_strcasecmp(lname, name) == 0 &&
-        g_strcasecmp(loption, option) == 0)
+    if (strcasecmp(lname, name) == 0 &&
+        strcasecmp(loption, option) == 0)
     {
       opt = strchr(line, ':') + 1;
       while (*opt == ' ' || *opt == '\t')
@@ -856,8 +744,3 @@ ppd_find(char *ppd_file,	/* I - Name of PPD file */
 
   return (NULL);
 }
-
-
-/*
- * End of "$Id: print-ps.c,v 1.19 2000/01/28 13:12:50 neo Exp $".
- */
